<?php

class Easy_Folders_Settings_Controller
{
    /**
     * Class instance
     * @since 1.0.0
    */
    protected static $instance = null;

    /**
     * Return class instance only
     * @since 1.0.0
     * @return object
    */
    public static function getInstance()
    {
        if (null == self::$instance) {
            self::$instance = new self();
        }
        return self::$instance;
    }

    /**
     * Settings api routes
     * @since 1.0.0
     * @return void
     */
    public function routes()
    {
        register_rest_route(
            EASY_FOLDERS_REST_API_PREFIX,
            'settings/load-settings',
            array(
                'methods' => 'GET',
                'callback' => array($this, 'index'),
                'permission_callback' => function () {
                    return current_user_can('upload_files');
                }
            )
        );

        register_rest_route(
            EASY_FOLDERS_REST_API_PREFIX,
            'settings/save-settings',
            array(
                'methods' => 'PUT',
                'callback' => array($this, 'saveSettings'),
                'permission_callback' => function () {
                    $capability = is_multisite() ? 'manage_network' : 'manage_options';
                    return current_user_can($capability);
                }
            )
        );

        

        register_rest_route(
            EASY_FOLDERS_REST_API_PREFIX,
            'settings/save-color',
            array(
                'methods' => 'POST',
                'callback' => array($this, 'saveColor'),
                'permission_callback' => function () {
                    $capability = is_multisite() ? 'manage_network' : 'manage_options';
                    return current_user_can($capability);
                }
            )
        );

        register_rest_route(
            EASY_FOLDERS_REST_API_PREFIX,
            'settings/delete-color',
            array(
                'methods' => 'POST',
                'callback' => array($this, 'deleteColor'),
                'permission_callback' => function () {
                    $capability = is_multisite() ? 'manage_network' : 'manage_options';
                    return current_user_can($capability);
                }
            )
        );

        register_rest_route(
            EASY_FOLDERS_REST_API_PREFIX,
            'settings/recreate-tables',
            array(
                'methods' => 'POST',
                'callback' => array($this, 'reCreateTable'),
                'permission_callback' => function () {
                    $capability = is_multisite() ? 'manage_network' : 'manage_options';
                    return current_user_can($capability);
                }
            )
        );

        register_rest_route(
            EASY_FOLDERS_REST_API_PREFIX,
            'settings/clear-all-data',
            array(
                'methods' => 'POST',
                'callback' => array($this, 'clearAllData'),
                'permission_callback' => function () {
                    $capability = is_multisite() ? 'manage_network' : 'manage_options';
                    return current_user_can($capability);
                }
            )
        );

        register_rest_route(
            EASY_FOLDERS_REST_API_PREFIX,
            'settings/get-site-health',
            array(
                'methods' => 'GET',
                'callback' => array($this, 'getSiteHealth'),
                'permission_callback' => function () {
                    $capability = is_multisite() ? 'manage_network' : 'manage_options';
                    return current_user_can($capability);
                }
            )
        );

        register_rest_route(
            EASY_FOLDERS_REST_API_PREFIX,
            'settings/add-newsletter-email',
            array(
                'methods' => 'POST',
                'callback' => array($this, 'addNewsletterEmail'),
                'permission_callback' => function () {
                    $capability = is_multisite() ? 'manage_network' : 'manage_options';
                    return current_user_can($capability);
                }
            )
        );

        register_rest_route(
            EASY_FOLDERS_REST_API_PREFIX,
            'settings/submit-feedback',
            array(
                'methods' => 'POST',
                'callback' => array($this, 'submitFeedback'),
                'permission_callback' => function () {
                    $capability = is_multisite() ? 'manage_network' : 'manage_options';
                    return current_user_can($capability);
                }
            )
        );

        

        register_rest_route(
            EASY_FOLDERS_REST_API_PREFIX,
            'settings/dismiss-notice',
            array(
                'methods' => 'POST',
                'callback' => array($this, 'dismissNotice'),
                'permission_callback' => '__return_true'
            )
        );

        register_rest_route(
            EASY_FOLDERS_REST_API_PREFIX,
            'settings/installation-step',
            array(
                'methods' => 'POST',
                'callback' => array($this, 'installationStep'),
                'permission_callback' => '__return_true'
            )
        );
    }

    /**
     * Return EasyFolders settings
     * @since 1.0.0
     * @return JSON
     */
    public function index()
    {
        $settings = (array) Easy_Folders_Settings::loadSettings();

        $settings['exts'] = (array) Easy_Folders_Settings::loadUploadFileTypes();

        $settings['active_language'] = Easy_Folders_Settings::getActiveTranslation();

        $settings['post_types'] = Easy_Folders_Helper::foldersForPostTypes();

        $settings['mime_types'] = get_allowed_mime_types();

        wp_send_json_success([
            'success' => true,
            'settings' => $settings
        ]);
    }

    

    /**
     * Save EasyFolders settings
     * @since 1.0.0
     * @param $request
     * @return JSON
     */
    public function saveSettings($request)
    {
        $input = Easy_Folders_Helper::sanitizeArray($request->get_param('settings'));

        $input = isset($input) ? Easy_Folders_Helper::sanitizeArray($input) : '';

        Easy_Folders_Settings::saveSettings($input);

        

        wp_send_json_success([
            'reload' => false,
            'success' => true,
            "message" => __('Settings saved successfully', 'easy-folders')
        ]);
    }

    /**
     * Add/update folder color
     * @since 1.0.0
     * @param $request
     * @return JSON
     */
    public function saveColor($request)
    {
        $color = sanitize_text_field($request->get_param('color'));
        $action = sanitize_text_field($request->get_param('action'));
        if ($action == "update") {
            $id = sanitize_text_field($request->get_param('id'));
            Easy_Folders_Settings::saveColor($color, $action, $id);
        } else {
            Easy_Folders_Settings::saveColor($color, $action);
        }
        wp_send_json_success();
    }

    /**
     * Delete folder color
     * @since 1.0.0
     * @param $request
     * @return JSON
     */
    public function deleteColor($request)
    {
        $id = sanitize_text_field($request->get_param('id'));
        Easy_Folders_Settings::deleteColor($id);
        wp_send_json_success();
    }

    /**
     * Check if required tables exist and create them if not.
     * @since 1.0.0
     * @param $request
     * @return JSON
     */
    public function reCreateTable($request)
    {
        Easy_Folders_Settings::reCreateTable();

        wp_send_json_success([
            'success' => true,
            "message" => __("Missing tables recreated successfully", 'easy-folders')
        ]);
    }

    /**
     * Wipe all data and reset to default factory settings.
     * @since 1.0.0
     * @param $request
     * @return JSON
     */
    public function clearAllData($request)
    {
        $status = Easy_Folders_Settings::clearAllData();

        if ($status) {
            wp_send_json_success(array(
                'success' => true,
                'message' => __('Successfully cleared', 'easy-folders')
            ));
        } else {
            wp_send_json_error(array(
                'success' => false,
                'message' => __('Please try again', 'easy-folders')
            ));
        }
    }

    /**
     * Monitor your system's status and see the details of key components.
     * @since 1.0.0
     * @param $request
     * @return JSON
     */
    public function getSiteHealth($request)
    {
        $response = Easy_Folders_Settings::getSiteHealth();

        $text = Easy_Folders_Settings::getSiteHealthCopyToClipboardData($response);

        wp_send_json_success([
            'success' => true,
            "response" => $response,
            "text" => $text
        ]);
    }

    /**
     * Save newsletter email in RipeBits account for marketing campaigns
     * @since 1.0.0
     * @param $request
     * @return JSON
     */
    public function addNewsletterEmail($request)
    {
        $email = sanitize_text_field($request->get_param('email'));

        //Remote api instance
        $api = new Easy_Folders_Api(Easy_Folders_Helper::getAccountApikey());

        $redirect = admin_url('admin.php');

        if (! is_email($email)) {
            wp_send_json_error([
                'message' => __('Invalid email address', 'easy-folders')
            ]);
        } else {
            $response = $api->addNewsLetterEmail($email, $redirect, true);

            if (is_wp_error($response)) {
                $code = is_numeric($response->get_error_code()) ? $response->get_error_code() : null;
                wp_send_json_error(
                    array(
                        'message' => $response->get_error_message(),
                    ),
                    $code
                );
            }

            $status = (object)json_decode($response['body']);

            // Too many requests.
            if (is_null($status) || wp_remote_retrieve_response_code($response) === 429) {
                wp_send_json_error(
                    array(
                        'message' =>  __('Too many requests, please try again in a moment', 'easy-folders')
                    ),
                    200
                );
            } else {
                if ($status->success) {
                    Easy_Folders_Helper::addOrUpdateOption(EASY_FOLDERS_PREFIX . 'newsletter-id', $status->data->id);
                    wp_send_json_success([
                        'message' => __("Great! Check your inbox to confirm your subscription", 'easy-folders')
                    ]);
                } else {
                    Easy_Folders_Settings::dismissNotice(EASY_FOLDERS_PREFIX . 'newsletter-hide');
                    wp_send_json_error([
                        'message' => is_array($status->message) ? implode(' ', $status->message) : $status->message
                    ]);
                }
            }
        }
    }

    /**
     * Submit feedback when deactivate plugin
     * @since 1.0.0
     * @param $request
     * @return JSON
     */
    public function submitFeedback($request)
    {
        $name = sanitize_text_field($request->get_param('name'));

        $email = sanitize_text_field($request->get_param('email'));

        $feedback = sanitize_text_field($request->get_param('feedback'));

        $rating = sanitize_text_field($request->get_param('rating'));

        $type = sanitize_text_field($request->get_param('type'));

        $option = sanitize_text_field($request->get_param('option'));

        $response = Easy_Folders_Settings::submitFeedback($rating);

        if ($response || $type == "feedback") {
            //Remote api instance
            $api = new Easy_Folders_Api(Easy_Folders_Helper::getAccountApikey());

            $response = $api->submitFeedback($email, $rating, $feedback, $name, $type, $option, true);

            if (is_wp_error($response)) {
                if ($type == "rating") {
                    Easy_Folders_Settings::deleteFeedback();
                }
                $code = is_numeric($response->get_error_code()) ? $response->get_error_code() : null;
                wp_send_json_error(
                    array(
                        'message' => $response->get_error_message(),
                    ),
                    $code
                );
            }

            $status = (object)json_decode($response['body']);

            // Too many requests.
            if (is_null($status) || wp_remote_retrieve_response_code($response) === 429) {
                if ($type == "rating") {
                    Easy_Folders_Settings::deleteFeedback();
                }
                wp_send_json_error(
                    array(
                        'message' =>  __('Too many requests, please try again in a moment', 'easy-folders')
                    ),
                    200
                );
            } else {
                if ($status->success) {
                    Easy_Folders_Helper::addOrUpdateOption($type == "feedback" ? EASY_FOLDERS_PREFIX . 'feedback-id' : EASY_FOLDERS_PREFIX . 'rating-id', $status->data->id);
                    wp_send_json_success([
                        'message' => __("Feedback submitted successfully", 'easy-folders')
                    ]);
                } else {
                    if ($type == "rating") {
                        Easy_Folders_Settings::deleteFeedback();
                    }
                    wp_send_json_error([
                        'message' => is_array($status->message) ? implode(' ', $status->message) : $status->message
                    ]);
                }
            }
        } else {
            wp_send_json_error(
                array(
                    'message' => __("Feedback already submitted", 'easy-folders')
                ),
                200
            );
        }
    }

    

    /**
     * This is generic function ti dismiss wordpress notices
     * @since 1.0.0
     * @param $request
     * @return JSON
     */
    public function dismissNotice($request)
    {
        $notice = sanitize_text_field($request->get_param('notice'));
        $parameter_1 = sanitize_text_field($request->get_param('parameter_1'));
        Easy_Folders_Settings::dismissNotice($notice, $parameter_1);
        wp_send_json_success();
    }

    /**
     * Save plugin installation step
     * @since 1.0.0
     * @param $request
     * @return JSON
     */
    public function installationStep($request)
    {
        $step = sanitize_text_field($request->get_param('step'));
        Easy_Folders_Settings::installationStep($step);
        wp_send_json_success();
    }
}
