<?php

class Easy_Folders_Helper
{
    /**
     * Class instance
     * @since 1.0.0
     * @var object $instance
    */
    protected static $instance = null;

    /**
     * Return class instance statically
     * @since    1.0.0
     * @return object
    */
    public static function getInstance()
    {
        if (null == self::$instance) {
            self::$instance = new self();
        }
        return self::$instance;
    }

    /**
     * Sanitize array input
     * @since    1.0.0
     * @var string $var
     * @return array
     */
    public static function sanitizeArray($var)
    {
        if (is_array($var)) {
            return array_map('self::sanitizeArray', $var);
        } else {
            return is_scalar($var) ? sanitize_text_field($var) : $var;
        }
    }

    /**
     * Return site url
     * @since    1.0.0
     * @return string
    */
    public static function wxrSiteUrl()
    {
        if (is_multisite()) {
            return network_home_url();
        } else {
            return get_bloginfo_rss('url');
        }
    }

    /**
     * Gets the RipeBits API key.
     * @since 1.0.0
     * @return string|false
     */
    public static function getAccountApikey()
    {
        $settings = Easy_Folders_Settings::loadSettings();
        return $settings['license_key'];
    }

    /**
     * Returns current user name to be displayed
     * @since 1.0.0
     * @return string
     */
    public static function getUserName()
    {
        $current_user = wp_get_current_user();
        return !empty($current_user->first_name) ? $current_user->first_name : $current_user->display_name;
    }

    /**
     * Additional available file types.
     * @since 1.0.0
     * @return array
     */
    public static function getAvailableFileTypes()
    {
        $file = 'file-types-list';

        $mimeTypes = trim(file_get_contents(EASY_FOLDERS_PLUGIN_DIR . '/admin/json/' . $file . '.json'));

        return json_decode($mimeTypes, true);
    }

    /**
     * Create DB structure
     * @since 1.0.0
     * @return void
     */
    public static function createDbStructure()
    {
        global $wpdb;

        $charset_collate = $wpdb->get_charset_collate();

        $table_efs = $wpdb->prefix . 'efs';

        //Set sql mode
        $wpdb->set_sql_mode(['ALLOW_INVALID_DATES']);

        //type == 0: default
        //type == 1: woocommerce
        if ($wpdb->get_var("show tables like '$table_efs'") != $table_efs) {
            $sql = 'CREATE TABLE ' . $table_efs . ' (
				`id` BIGINT(20) unsigned NOT NULL AUTO_INCREMENT,
                `wocommerce_parent` TINYINT(1) NULL DEFAULT 0,
				`name` varchar(250) NULL,
				`parent` BIGINT(20) NULL DEFAULT 0,
				`type` int(2) NULL DEFAULT 0,
				`ord` BIGINT(20) NULL DEFAULT 0,
				`color` varchar(30) NULL,
				`post_type` varchar(25) NULL,
				`starred` TINYINT(1) NULL DEFAULT 0,
				`created_by` BIGINT(20) NULL DEFAULT 0,
				`term_id` BIGINT(20) NULL DEFAULT 0,
				`product_id` BIGINT(20) NULL DEFAULT 0,
				`shortcut` BIGINT(20) NULL DEFAULT 0,
				`created_at` TIMESTAMP NULL DEFAULT CURRENT_TIMESTAMP,
				`updated_at` TIMESTAMP on update CURRENT_TIMESTAMP NULL DEFAULT CURRENT_TIMESTAMP,
				`deleted_at` DATETIME NULL DEFAULT NULL,
				PRIMARY KEY (id),
				UNIQUE KEY `id` (id)) ' . 'ENGINE = InnoDB ' . $charset_collate . ';';
            require_once ABSPATH . 'wp-admin/includes/upgrade.php';
            dbDelta($sql);
        }

        $table = $wpdb->prefix . 'efs_attachment_folder';

        //type == 0: default
        //type == 1: woocommerce

        if ($wpdb->get_var("show tables like '$table'") != $table) {
            $sql = 'CREATE TABLE ' . $table . ' (
				`folder_id` BIGINT(20) unsigned NOT NULL,
				`attachment_id` bigint(20) unsigned NOT NULL,
				`product_id` BIGINT(20) NULL DEFAULT 0,
				`restore` TINYINT(1) NULL DEFAULT 0,
				`created_at` TIMESTAMP NULL DEFAULT CURRENT_TIMESTAMP,
				`updated_at` TIMESTAMP on update CURRENT_TIMESTAMP NULL DEFAULT CURRENT_TIMESTAMP,
				`deleted_at` DATETIME NULL DEFAULT NULL,
				UNIQUE( `folder_id`, `attachment_id`)
				) ' . 'ENGINE = InnoDB ' . $charset_collate . ';';
            require_once ABSPATH . 'wp-admin/includes/upgrade.php';
            dbDelta($sql);
        }

        $efs_upload_file_types = $wpdb->prefix . 'efs_upload_file_types';

        if ($wpdb->get_var("show tables like '$efs_upload_file_types'") != $efs_upload_file_types) {
            $sql = 'CREATE TABLE ' . $efs_upload_file_types . ' (
				`id` BIGINT(20) unsigned NOT NULL AUTO_INCREMENT,
				`description` varchar(500) NULL,
				`mime_type` varchar(100) NULL,
				`ext` varchar(50) NULL,
				`status` TINYINT(1) NULL DEFAULT 0,
				PRIMARY KEY (id),
				UNIQUE KEY `id` (id)) ' . 'ENGINE = InnoDB ' . $charset_collate . ';';
            require_once ABSPATH . 'wp-admin/includes/upgrade.php';
            dbDelta($sql);
        }

        $efs_colors = $wpdb->prefix . 'efs_colors';

        if ($wpdb->get_var("show tables like '$efs_colors'") != $efs_colors) {
            $sql = 'CREATE TABLE ' . $efs_colors . ' (
				`id` INT(11) unsigned NOT NULL AUTO_INCREMENT,
				`color` varchar(22) NULL,
				PRIMARY KEY (id),
				UNIQUE KEY `id` (id)) ' . 'ENGINE = InnoDB ' . $charset_collate . ';';
            require_once ABSPATH . 'wp-admin/includes/upgrade.php';
            dbDelta($sql);
        }

        $current_version = get_option('easyfolders_version');

        if (version_compare(EASY_FOLDERS_VERSION, $current_version, '>')) {
            update_option('easyfolders_version', EASY_FOLDERS_VERSION);
        }

        //Insert default data for folders
        $query = $wpdb->prepare('SELECT * FROM %1$s WHERE `id` = "1"', $table_efs);

        $starred = $wpdb->get_row($query);

        if (is_null($starred)) {
            $wpdb->insert(
                $table_efs,
                array(
                    'id' => 1,
                    'name' => 'Starred',
                ),
                array('%d', '%s')
            );
        }

        $query = $wpdb->prepare('SELECT * FROM %1$s WHERE `id` = "2"', $table_efs);

        $trashed = $wpdb->get_row($query);

        if (is_null($trashed)) {
            $wpdb->insert(
                $table_efs,
                array(
                    'id' => 2,
                    'name' => 'Trashed',
                ),
                array('%d', '%s')
            );
        }

        Easy_Folders_Admin::registerSchedules();
    }

    /**
     * Reset default settings
     * @since 1.0.0
     * @param $column
     * @param $free
     * @return void
    */
    public static function resetDefaultSettings($install = false)
    {
        $default_options = [
            'media_rename' => 1,
            'media_replace' => 1,
            'media_download' => 1,
            'folder_color' => 1,
            'trash_bin' => 1,
            'user_folder' => 1,
            'folder_lock' => 1,
            'woocommerce_support' => 1,
            'gutenberg_support' => 0,
            'thirdparty_compatibility' => 1,
            'gutenberg_editor_support' => 1,
            'woocommerce_editor_support' => 1,
            'class_editor_support' => 1,
            'elementor_support' => 1,
            'beaver_support' => 1,
            'wpbakery_support' => 1,
            'brizy_support' => 1,
            'cornerstone_support' => 1,
            'divi_support' => 1,
            'thrive_quiz_support' => 1,
            'fusion_support' => 1,
            'oxygen_support' => 1,
            'tatsu_support' => 1,
            'dokan_support' => 1,
            'max_file_upload_size' => 20,
            'max_file_upload_default' => 1,
            'file_types' => 0,
            'language' => 'en',
            
            'usage_tracking' => 0,
            'is_plugin_removal_delete_data' => 0,
            'access_setting' => 0,
            'starred_media' => 1,
            'starred_folder' => 1,
            'starred_attachment' => 1,
            'auto_delete_old_items' => 0,
            'auto_delete_item_after_days' => 30,
            'is_folder_media_deleted' => 1,
            'woocommerce_sub_folder_creation_by' => 'name',
            'rename_media_type_values' => [],
            'rename_media_type' => 'all',
            'rename_media_remove_char' => [],
            'auto_media_format' => 1,
            'auto_media_rename' => 1,
            'auto_media_format_text' => 1,
            'media_hover_detail' => 1,
            'media_hover_detail_list' => ['title', 'dimension', 'size'],
            'infinite_scrolling' => 0,
        ];

        foreach ($default_options as $option => $value) {
            if($install) {
                $record = Easy_Folders_Helper::existOption(EASY_FOLDERS_PREFIX. $option);
                if(!$record) {
                    Easy_Folders_Helper::addOrUpdateOption(EASY_FOLDERS_PREFIX . $option, $value);
                }
            } else {
                if (!in_array($option, ['license_key', 'site_status'])) {
                    Easy_Folders_Helper::addOrUpdateOption(EASY_FOLDERS_PREFIX . $option, $value);
                }
            }
        }

        

        

        //Reset default colors
        Easy_Folders_Settings::resetDefaultColors();

        //Folders support for custom post types
        $post_types = Easy_Folders_Helper::foldersForPostTypes();

        if($post_types && is_array($post_types) && count($post_types) > 0) {
            foreach ($post_types as $key => $post_type) {
                Easy_Folders_Helper::addOrUpdateOption(EASY_FOLDERS_PREFIX . $post_type->name, 1);
            }
        }

        //Save custom post types
        Easy_Folders_Helper::addOrUpdateOption(EASY_FOLDERS_PREFIX . 'custom-post-types', array_column($post_types, 'name'));
    }

    /**
     * Settings options
     * @since 1.0.0
     * @return array
     */
    public static function settingsOptions()
    {
        return ['media_rename', 'media_replace', 'media_download', 'starred_media', 'folder_color', 'trash_bin', 'user_folder', 'folder_lock', 'woocommerce_support', 'gutenberg_editor_support', 'woocommerce_editor_support', 'class_editor_support', 'elementor_support', 'beaver_support', 'wpbakery_support', 'brizy_support', 'cornerstone_support', 'divi_support', 'thrive_quiz_support', 'fusion_support', 'oxygen_support', 'tatsu_support', 'dokan_support', 'thirdparty_compatibility', 'gutenberg_support', 'max_file_upload_size', 'max_file_upload_default', 'file_types', 'language', 'usage_tracking', 'is_plugin_removal_delete_data', 'access_setting', 'license_key', 'site_status', 'starred_folder', 'starred_attachment', 'auto_delete_item_after_days', 'woocommerce_sub_folder_creation_by', 'rename_media_type_values', 'rename_media_type', 'rename_media_remove_char', 'auto_media_format', 'auto_media_rename', 'auto_media_format_text', 'is_folder_media_deleted', 'auto_delete_old_items', 'media_hover_detail', 'media_hover_detail_list', 'infinite_scrolling'];
    }

    

    /**
     * Add/update wordpress option value
     * @since 1.0.0
     * @param string $key
     * @param string $value
     * @return void
    */
    public static function addOrUpdateOption($key, $value)
    {
        $option = self::existOption($key);
        if ($option) {
            update_option($key, $value);
        } else {
            add_option($key, $value);
        }
    }

    /**
     * Check option key exist or not
     * @since 1.0.0
     * @param string $arg
     * @return boolean
    */
    public static function existOption( $arg ) {
        global $wpdb;
        $prefix = $wpdb->prefix;
        $db_options = $prefix.'options';
        $sql_query = 'SELECT * FROM ' . $db_options . ' WHERE option_name = "' . $arg . '"';
        $option = $wpdb->get_row( $sql_query);
        if ( is_null($option) ) {
            return false;
        } else {
            return true;
        }
    }

    /**
     * Geo location info
     * @since 1.0.0
     * @return array
     */
    public static function getLocationInfoByIp()
    {
        $client  = sanitize_text_field(@$_SERVER['HTTP_CLIENT_IP']);
        $forward = sanitize_text_field(@$_SERVER['HTTP_X_FORWARDED_FOR']);
        $remote  = sanitize_text_field(@$_SERVER['REMOTE_ADDR']);
        $result  = array('country'=>'', 'city'=>'');
        if (filter_var($client, FILTER_VALIDATE_IP)) {
            $ip = $client;
        } elseif (filter_var($forward, FILTER_VALIDATE_IP)) {
            $ip = $forward;
        } else {
            $ip = $remote;
        }
        $ip_data = @json_decode(file_get_contents("http://www.geoplugin.net/json.gp?ip=".$ip));
        if ($ip_data && $ip_data->geoplugin_countryName != null) {
            $result['country'] = $ip_data->geoplugin_countryCode;
            $result['city'] = $ip_data->geoplugin_city;
        }
        return $result;
    }

    /**
     * Get domain
     * @since 1.0.0
     * @param string $url
     * @return string
     */
    public static function getDomain($url)
    {
        if (substr($url, 0, 7) == 'http://') {
            return $url;
        }

        if (substr($url, 0, 8) == 'https://') {
            return $url;
        }

        $site_url_parts = parse_url(get_site_url());

        if (isset($site_url_parts['scheme']) && $site_url_parts['scheme'] && $site_url_parts['scheme'] == 'https') {
            return 'https://'.$url;
        } else {
            return 'http://'.$url;
        }
    }

    /**
     * Get host name
     * @since 1.0.0
     * @param mixed $url
     * @return string
     */
    public static function getHostname($url)
    {
        $parse = parse_url($url);
        if (isset($parse['host']) && $parse['host']) {
            return str_ireplace('www.', '', $parse['host']);
        } else {
            return $url;
        }
    }

    /**
     * Get current user role
     * @since 1.0.0
     * @return string
    */
    public static function getCurrentUserRole()
    {
        $role = null;

        $current_user = wp_get_current_user();

        if (! empty($current_user->roles)) {
            $role = $current_user->roles[0];
        }

        return $role;
    }

    /**
     * Sanitize input
     * @since 1.0.0
     * @param $arr
     * @return array|string
     */
    public static function sanitizeTextOrArrayField($array_or_string)
    {
        if (is_string($array_or_string)) {
            $array_or_string = sanitize_text_field($array_or_string);
        } elseif (is_array($array_or_string)) {
            foreach ($array_or_string as $key => &$value) {
                if (is_array($value)) {
                    $value = self::sanitizeTextOrArrayField($value);
                } else {
                    $value = sanitize_text_field($value);
                }
            }
        }

        return $array_or_string;
    }

    /**
     * Return current page post type
     * @since 1.0.0
     * @return string
     */
    public static function get_custom_post_type()
    {
        global $typenow;
        global $pagenow;
        if ($pagenow === 'edit.php') {
            if (empty($typenow)) $typenow = 'post';
            $post_type = $typenow;
        } else if (in_array($pagenow, ['media-new.php', 'upload.php'])) {
            $post_type = 'attachment';
        } else {
            $post_type = $typenow;
        }
        return $post_type;
    }

    /**
     * Get use folders with post types
     * @since 1.0.0
     * @return array
     */
    public static function foldersForPostTypes()
    {
        $post_types = array();

        $custom_post_types = (array)get_post_types( array( ), 'objects' );

        foreach ($custom_post_types as $key => $custom_post_type) {
            if ( ! $custom_post_type->show_ui) continue;
            $post_types[$key] = $custom_post_type;
            $post_types[$key]->free = in_array($custom_post_type->name, ['attachment', 'post', 'page']) ? 1 : 0;
        }

        //Save custom post types
        Easy_Folders_Helper::addOrUpdateOption(EASY_FOLDERS_PREFIX . 'custom-post-types', array_column($post_types, 'name'));

        return $post_types;
    }

    /**
     * Rebuild the query url for specified param
     * @since 1.0.0
     * @param $url
     * @param $key
     * @param $value
     * @return string
     */
    public static function add_url_query_arg($url, $key, $value) {
	
        $url = preg_replace('/(.*)(?|&)'. $key .'=[^&]+?(&)(.*)/i', '$1$2$4', $url .'&');
        $url = substr($url, 0, -1);
        
        if (strpos($url, '?') === false) {
            return ($url .'?'. $key .'='. $value);
        } else {
            return ($url .'&'. $key .'='. $value);
        }
    }
}
