<?php

/**
 * Easy Folders Media library class.
 * Responsible for displaying a UI (stats + action links) in the media library and the editor.
 * @since 1.0.0
 */

class Easy_Folders_Media_Hook
{
    /**
     * Folder table
     * @since 1.0.0
     * @var $folder_table
    */
    private static $folder_table = 'efs';

    /**
     * Folder relation table
     * @since 1.0.0
     * @var $relation_table
    */
    private static $relation_table = 'efs_attachment_folder';

    /**
     * settings
     * @since 1.0.0
     * @var $settings
    */
    private $settings;

    /**
     * To detect User wise media | User Folders
     * @since 1.0.0
     * @var boolean $is_enabled
    */
    private $is_enabled;

    /**
     * Current user media
     * @since 1.0.0
     * @var int $current_user_id
    */
    private $current_user_id;

    /**
     * Media_Library constructor.
     * @since 1.0.0
     * @return void
     */
    public function __construct($settings)
    {
        $this->settings = $settings;
        $this->current_user_id = get_current_user_id();
        $this->is_enabled = $this->isEnabled();
    }

    /**
     * User Folders settings
     * @since 1.0.0
     * @param $data
     * @return array
    */
    private function isEnabled()
    {
        // For now this feature is disabled
        return false;
    }
    
    /**
     * Extend wp hooks
     * @since 1.0.0
     * @return void
    */
    public function wp_hooks()
    {
        global $typenow;

        // Current post type
        $current_post_type =  Easy_Folders_Helper::get_custom_post_type();

        //Folders support for custom post types
        $post_types = get_option(EASY_FOLDERS_PREFIX . 'custom-post-types', []);
        
        // Media 
        if(in_array('attachment', $post_types) && $this->settings['attachment'] == 1 && $current_post_type == 'attachment') {
            add_filter('manage_media_columns', array($this, 'media_columns_head'));
            add_action('manage_media_custom_column', array($this, 'manage_columns_content'), 10, 2);
        }

        // Pages 
        if(in_array('page', $post_types) && $this->settings['page'] == 1 && $current_post_type == 'page') {
            add_filter('manage_page_posts_columns', array($this, 'media_columns_head'));
            add_action('manage_page_posts_custom_column', array($this, 'manage_columns_content'), 10, 2);
        }

        // Posts 
        if(in_array('post', $post_types) && $this->settings['post'] == 1 && $current_post_type == 'post') {
            add_filter('manage_posts_columns', array($this, 'media_columns_head'));
            add_action('manage_posts_custom_column', array($this, 'manage_columns_content'), 10, 2);
        }

        

        add_filter('manage_upload_sortable_columns', array($this, 'sortable_column'));

        // Add pre WordPress 5.0 compatibility.
        add_filter('wp_kses_allowed_html', array($this, 'filter_html_attributes'));

        

        //Media relations
        add_filter('wp_prepare_attachment_for_js', array($this, 'media_relation'), 99, 5);

        

        
        
        // Fires before the Filter button on the Posts and Pages list tables.
		add_action( 'restrict_manage_posts', array( $this, 'add_filter_dropdown' ) );

        // Filters the list of available list table views.
        add_filter('views_edit-' . $typenow, array($this, 'filter_available_list_views_links'), 99, 1);

        

    }

    /**
     * My hooks
     * @since 1.0.0
     * @return void
    */
    public function my_hooks()
    {
        if ($this->is_enabled) {

            add_filter('easyfolders_pre_creating_folders', array($this, 'filterAttributesBeforeInsertingFolder'));

            add_filter('easyfolders_current_user_folders_where', array($this, 'currentUserFolders'), 10, 2);

            add_filter('easyfolders_current_user_id', array($this, '_current_user_id'));
        }

        add_action('efs_action_before_folder_media_relation', array($this, 'actionBeforeFolderMediaRelation'), 10, 2);
    }

    /**
     * Print column header for Compress results in the media library
     * @since 1.0.0
     * @param array $defaults  Defaults array.
     * @return array
     */
    public function media_columns_head($defaults)
    {
        $custom = array(
            'drag_icon' 	=> '<span class="easy-folders-move-file dashicons dashicons-move"></span>',

        );

        $defaults = $custom + $defaults;

        return $defaults;
    }

    /**
     * Add the sortable column to sortable list
     * @since 1.0.0
     * @param array $columns
     * @return array
     */
    public function sortable_column($columns)
    {
        return $columns;
    }

    /**
     * Print column data for Compress results in the media library
     * @since 1.0.0
     * @param string $column_name
     * @param int $id 
     */
    public function manage_columns_content($column_name, $id)
    {
        if ('drag_icon' === $column_name) {
            $attachment = get_post($id);
            $attachment = array('id' => $attachment->id, 'post_date' => esc_attr($attachment->post_date), 'post_title' => esc_attr($attachment->post_title), 'post_name' => esc_attr($attachment->post_name), 'guid' => esc_attr($attachment->guid), 'post_status' => esc_attr($attachment->post_status));
            $attachment = $attachment && is_array($attachment) ? json_encode($attachment, true) : [];
            echo wp_kses_post("<span data-id='".$id."' class='easy-folders-move-file dashicons dashicons-move'></span>
            <input type='hidden' class='attachment-info-".$id."' value='".$attachment."' />");
        }
    }

    /**
     * Some data attributes are not allowed on <a> tag/elements on WordPress < 5.0.0.
     * @since 1.0.0
     * @param array $context
     * @return mixed
     */
    public function filter_html_attributes($context)
    {
        global $wp_version;

        if (version_compare('5.0.0', $wp_version, '<')) {
            return $context;
        }

        $context['a']['data-tooltip'] = true;

        $context['a']['data-id']      = true;

        $context['a']['data-nonce']   = true;

        return $context;
    }

    

    

    

    

    

    

    

    

    

    /**
     * Media relations
     * @since 1.0.0
     * @param $response
     * @param $attachment
     * @return array
     */
    public function media_relation($response, $attachment)
    {
        global $wpdb;

        if (!isset($attachment->ID) && !isset($attachment->id)) {
            return $response;
        }

        $attachment_id = isset($attachment->ID) ? $attachment->ID : $attachment->id;

        if ((int)$attachment->post_parent > 0) {
            $response['post_parent'] = $attachment->post_parent;
            $post = get_post($attachment->post_parent);
            if ($post) {
                $response['attached_post_title'] = $post->post_title;
            }
        }

        $relation = $wpdb->get_row($wpdb->prepare('SELECT * FROM %1$s WHERE `attachment_id` = %2$d AND deleted_at IS NULL', self::getTable(self::$relation_table), $attachment_id));

        $response['folder_id'] = $relation ? $relation->folder_id : 0;

        $response['starred'] = get_post_meta((int)$attachment_id, EASY_FOLDERS_PREFIX . 'starred', true);

        $response['deleted_at'] = $attachment->post_modified;

        $caption = '';

        $media_fields = $this->settings['media_hover_detail'] && count((array)$this->settings['media_hover_detail_list']) > 0 ? $this->settings['media_hover_detail_list'] : [];
        
        $translations = Easy_Folders_i18n::getTranslation();

        if(count($media_fields) > 0) {

            $caption = '<div class="attachment-preview__details">';

            if(in_array('title', $media_fields)) {
                $caption.= '<p>'.$translations['title'].': '.$attachment->post_title.'</p>';
            }
            
            if(in_array('alternative_text', $media_fields)) {
                $caption.= '<p>'.$translations['alternative_text'].': '.get_post_meta($attachment_id , '_wp_attachment_image_alt', true).'</p>';
            }
            
            if(in_array('file_url', $media_fields)) {
                $caption.= '<p>'.$translations['alternative_text'].': '.$attachment->guid.'</p>';
            }
            
            if(in_array('dimension', $media_fields)) {
                $attachment_meta  =wp_get_attachment_metadata($attachment_id);
                if(isset($attachment_meta) && !empty($attachment_meta)) {
                    $caption.= '<p>'.$translations['dimensions'].': '.$attachment_meta['width'].' x '.$attachment_meta['height'].'</p>';
                }
            }
    
            if(in_array('size', $media_fields)) {
                $caption.= '<p>'.$translations['size'].': '.size_format(filesize( get_attached_file( $attachment->ID ) ), 0).'</p>';
            }
            
            if(in_array('filename', $media_fields)) {
                $caption.= '<p>'.$translations['filename'].': '.$attachment->post_name.'</p>';
            }
            
            if(in_array('type', $media_fields)) {
                $caption.= '<p>'.$translations['type'].': '.$attachment->post_mime_type.'</p>';
            }
            
            if(in_array('date', $media_fields)) {
                $caption.= '<p>'.$translations['date'].': '.date_i18n( get_option( 'date_format' ), strtotime( $attachment->post_date )).'</p>';
            }
            
            if(in_array('uploaded_by', $media_fields)) {
                $caption.= '<p>'.$translations['uploaded_by'].': '.get_the_author_meta('display_name', $attachment->post_author).'</p>';
            }

            $caption .= "</div>";
        }
        
        $response['preview_detail'] = $caption;

        return $response;
    }

    

    

    /**
     * Current user folders query
     * @since 1.0.0
     * @param $where
     * @param $folder_table
     * @return string
    */
    public function currentUserFolders($where, $folder_table)
    {
        return sprintf('`folder_id` IN (SELECT `id` FROM %1$s WHERE `created_by` = %2$d)', $folder_table, $this->current_user_id);
    }
    
    /**
     * Current user id
     * @since 1.0.0
     * @return int
    */
    public function _current_user_id()
    {
        return $this->current_user_id;
    }
    
    /**
     * Apply "User Folders" settings before inserting into folders 
     * @since 1.0.0
     * @param $post_id
     * @param $folder_id
     * @return object
    */
    public function actionBeforeFolderMediaRelation($post_id, $folder_id)
    {
        global $wpdb;
        
        if ($this->is_enabled) {
            $query = sprintf('DELETE FROM %1$sefs_attachment_folder WHERE `attachment_id` = %2$d AND `folder_id` IN (SELECT `id` FROM %1$sefs WHERE `created_by` = %3$d)', $wpdb->prefix, $post_id, $this->current_user_id);
            $wpdb->query($wpdb->prepare($query));
        } else {
            $wpdb->query(sprintf('DELETE FROM %1$sefs_attachment_folder WHERE `attachment_id` = %2$d AND `folder_id` IN (SELECT `id` FROM %1$sefs WHERE `created_by` = 0)', $wpdb->prefix, $post_id));
        }
    }

    /**
     * Filter before data inserted in DB
     * @since 1.0.0
     * @param $data
     * @return array
    */
    public function filterAttributesBeforeInsertingFolder($data)
    {
        $data['created_by'] = $this->current_user_id;
        
        return $data;
    }
    
    /**
	 * Adds a search dropdown in Media Library list view to filter out images
	 * @since 1.0.0
     * @return void
	 */
	public function add_filter_dropdown() {
        $orderby = filter_input( INPUT_GET, 'orderby', FILTER_SANITIZE_SPECIAL_CHARS );

        $order = filter_input( INPUT_GET, 'order', FILTER_SANITIZE_SPECIAL_CHARS );

        ?>
        <select class="attachment-filters" name="orderby" id="orderby">
            <option value="" <?php selected( $orderby, '' ); ?>><?php esc_html_e( 'Sort by', 'easy-folders' ); ?></option>
            <option value="name" <?php selected( $orderby, 'name' ); ?>><?php esc_html_e( 'Name', 'easy-folders' ); ?></option>
            <option value="date" <?php selected( $orderby, 'date' ); ?>><?php esc_html_e( 'Date', 'easy-folders' ); ?></option>
            <option value="modified" <?php selected( $orderby, 'modified' ); ?>><?php esc_html_e( 'Date Modified', 'easy-folders' ); ?></option>
            <option value="author" <?php selected( $orderby, 'author' ); ?>><?php esc_html_e( 'Author', 'easy-folders' ); ?></option>
        </select>
        <select class="attachment-filters" name="order" id="order">
            <option value="" <?php selected( $order, '' ); ?>><?php esc_html_e( 'Order by', 'easy-folders' ); ?></option>
            <option value="asc" <?php selected( $order, 'asc' ); ?>><?php esc_html_e( 'Ascending', 'easy-folders' ); ?></option>
            <option value="desc" <?php selected( $order, 'desc' ); ?>><?php esc_html_e( 'Descending', 'easy-folders' ); ?></option>
        </select>
        <?php
	}

    /**
	 * Filters the list of available list table views.
	 * @since 1.0.0
     * @param $views
     * @return array
	 */
	public function filter_available_list_views_links($views) {  

        global $wpdb, $wp_query;

        $sql = $wp_query->request;

        if (str_contains($sql, "wp_posts.post_status = 'trash'")) { 
            $sql = str_replace("wp_posts.post_status = 'trash'", "(wp_posts.post_status = 'publish' OR wp_posts.post_status = 'future' OR wp_posts.post_status = 'draft' OR wp_posts.post_status = 'pending' OR wp_posts.post_status = 'private' OR wp_posts.post_status = 'trash')", $sql);
        } else if(str_contains($sql, "wp_posts.post_status = 'publish'")) {
            $sql = str_replace("wp_posts.post_status = 'publish'", "wp_posts.post_status = 'publish' OR wp_posts.post_status = 'future' OR wp_posts.post_status = 'draft' OR wp_posts.post_status = 'pending' OR wp_posts.post_status = 'private' OR wp_posts.post_status = 'trash'", $sql);
        } else if(str_contains($sql, "wp_posts.post_status = 'future'")) {
            $sql = str_replace("wp_posts.post_status = 'future'", "wp_posts.post_status = 'publish' OR wp_posts.post_status = 'future' OR wp_posts.post_status = 'draft' OR wp_posts.post_status = 'pending' OR wp_posts.post_status = 'private' OR wp_posts.post_status = 'trash'", $sql);
        } else if(str_contains($sql, "wp_posts.post_status = 'pending'")) {
            $sql = str_replace("wp_posts.post_status = 'pending'", "wp_posts.post_status = 'publish' OR wp_posts.post_status = 'future' OR wp_posts.post_status = 'draft' OR wp_posts.post_status = 'pending' OR wp_posts.post_status = 'private' OR wp_posts.post_status = 'trash'", $sql);
        } else if(str_contains($sql, "wp_posts.post_status = 'draft'")) {
            $sql = str_replace("wp_posts.post_status = 'draft'", "wp_posts.post_status = 'publish' OR wp_posts.post_status = 'future' OR wp_posts.post_status = 'draft' OR wp_posts.post_status = 'pending' OR wp_posts.post_status = 'private' OR wp_posts.post_status = 'trash'", $sql);
        }

        $post_type = Easy_Folders_Helper::get_custom_post_type();
        
        $query = get_option('efs-query-params-'.$post_type) ? get_option('efs-query-params-'.$post_type) : [];

        $efs = isset($query['efs']) ? $query['efs'] : '';

        if ( strpos( $sql, 'FROM' ) !== false ) {
            list ( , $sql ) = explode( 'FROM', $sql );
        }
        if ( strpos( $sql, 'GROUP' ) !== false ) {
            list ( $sql ) = explode( 'GROUP', $sql );
        }
        if ( strpos( $sql, 'ORDER' ) !== false ) {
            list ( $sql ) = explode( 'ORDER', $sql );
        }

        $data = $wpdb->get_results( "SELECT post_status, count(ID) AS count FROM $sql GROUP BY post_status" );

        $total = array_sum( wp_list_pluck( $data, 'count' ) );

        //Reset count of all status
        if(count($views) > 0) {
            foreach ($views as $key => $view) {
                $views[$key] = preg_replace( '/\(.+\)/U', '(0)', $view );
            }
        }

        if(isset($views['all']) && $post_type) {
            $views[ 'all' ] = sprintf(
                '<a href="%s" class="%s">%s <span class="count">(%d)</span></a>',
                esc_url( admin_url( "edit.php?post_type={$post_type}" ) ),
                empty( $_GET['post_status'] ) ? 'current' : '',
                'All',
                $total
            );
        }

        $labels = [
            'publish' => 'Publish',
            'draft'   => 'Draft',
            'trash'   => 'Trash',
        ];

        foreach ( $data as $item ) {
            if(isset($labels[ $item->post_status ])) {
                $label = $labels[ $item->post_status];
                $views[ $item->post_status ] = sprintf(
                    '<a href="%s" class="%s">%s <span class="count">(%d)</span></a>',
                    esc_url( admin_url( "edit.php?post_type={$post_type}&post_status={$item->post_status}" ) ),
                    isset( $_GET['post_status'] ) && $_GET['post_status'] === $item->post_status ? 'current' : '',
                    $label,
                    $item->count
                );
            }
        }

        if(count($views) > 0 && $query && $efs) {
            foreach ($views as $key => $view) {
                preg_match_all('~<a(.*?)href="([^"]+)"(.*?)>~', $view, $matches);
                if(isset($matches[2]) && $matches[2]) {
                    $href = $matches[2];
                    if($href && is_array($href) && count($href) > 0) {
                        if($key == 'trash') {
                            $link = Easy_Folders_Helper::add_url_query_arg($href[0], 'efs', 2);
                            $link = Easy_Folders_Helper::add_url_query_arg($link, 'attachment-filter', 'trash');
                        } else {
                            $link = Easy_Folders_Helper::add_url_query_arg($href[0], 'efs', ($query['efs'] == 2 ? -1 : $query['efs']));
                        }
                        $link = str_replace($href[0], $link, $view);
                        $views[$key] = $link;
                    }
                }
            }
        }
        
        return $views;
    }

    /**
     * Mysql table
     * @since 1.0.0
     * @return void
     */
    private static function getTable($table)
    {
        global $wpdb;
        return $wpdb->prefix . $table;
    }
    
}
