<?php
/**
 * Class that communicate to RipeBits
 */
class Easy_Folders_Api
{
    /**
     * API key.
     * @since 1.0.0
     * @var string
     */
    public $license_key = '';

    /**
     * API request instance.
     * @since 1.0.0
     * @var Request
     */
    protected $request;

    /**
     * Endpoint name.
     * @since 1.0.0
     * @var string
     */
    public $name = 'ripebits';

    /**
     * Endpoint version.
     * @since 1.0.0
     * @var string
     */
    public $version = 'v1';

    /**
     * API constructor.
     * @since 1.0.0
     * @param string $key  API key.
     * @throws Exception  API Request exception.
     */
    public function __construct($key)
    {
        $this->license_key = $key;

        // The Request class needs these to make requests.
        if (empty($this->version) || empty($this->name)) {
            throw new Exception(__('API object require a version and name property', 'easy-folders'), 404);
        }

        $this->request = new Easy_Folders_Request($this);
    }
    
    /**
     * Fetch api status
     * @since 1.0.0
     * @param bool $is  Exponential back off.
     * @return mixed|WP_Error
     */
    public function get_site_status($is = false)
    {
        $response = $this->request->post_request(
            'subscription/validate-subscription',
            array(
                'license_key' => $this->license_key,
                'url'  => $this->request->get_this_site(),
            ),
            $is
        );
        
        //Unauthorized
        if(wp_remote_retrieve_response_code($response) == 402) {
            Easy_Folders_Settings::removeCredentials();
        }


        //Invalid domain
        if(wp_remote_retrieve_response_code($response) == 410) {
            $status = json_decode($response['body']);
            Easy_Folders_Settings::removeCredentials();
            Easy_Folders_Helper::addOrUpdateOption(EASY_FOLDERS_PREFIX . 'domain-mismatch', $status->data->domain);
        }
        
        return $response;
    }

    /**
     * Add newsletter email
     * @since 1.0.0
     * @param string $email  
     * @param bool $is Exponential back off
     * @return mixed|WP_Error
     */
    public function addNewsLetterEmail($email, $redirect, $is = false)
    {
        return $this->request->post_request(
            'user/add-newsletter-email',
            array(
                'email' => $email,
                'redirect' => $redirect,
            ),
            $is
        );
    }

    /**
     * Submit feedback
     * @since 1.0.0
     * @param string $email  
     * @param int $rating  
     * @param string $feedback  
     * @param string $name  
     * @param bool $is Exponential back off
     * @return mixed|WP_Error
     */
    public function submitFeedback($email, $rating, $feedback, $name, $type, $option, $is = false)
    {
        return $this->request->post_request(
            'user/feedback',
            array(
                'slug' => EASY_FOLDERS_PLUGIN_NAME,
                'email' => $email,
                'rating' => $rating,
                'feedback' => $feedback,
                'name' => $name,
                'type' => $type,
                'option' => $option,
            ),
            $is
        );
    }

    /**
     * Save usage tracking
     * @since 1.0.0
     * @param array $data  
     * @param string $domain  
     * @param bool $is Exponential back off
     * @return mixed|WP_Error
    */
    public function saveUsageTracking($data, $domain, $is = false)
    {
        return $this->request->put(
            'user/save-usage-tracking',
            array(
                'data' => $data,
                'domain' => $domain,
            ),
            $is
        );
    }

    /**
     * Return valid download plugin url
     * @since 1.0.0
     * @param  mixed $license_key
     * @return mixed
     */
    public function downloadPluginUrl($license_key = "")
    {
        return $this->request->getRequestUrl(
            $license_key ? 'subscription/download-plugin/'.EASY_FOLDERS_PLUGIN_NAME.'/'.$license_key : 'subscription/download-plugin'
        );
    }

    /**
     * Disconnect website
     * @since 1.0.0
     * @param array $website_id  
     * @param bool $is Exponential back off
     * @return mixed|WP_Error
    */
    public function disconnect($website_id, $is = false)
    {
        return $this->request->post_request(
            'subscription/remove-website',
            array(
                'license_key' => $this->license_key,
                'website_id' => $website_id,
            ),
            $is
        );
    }
}
