<?php
/**
 * In this class, you will find all generic settings related to Easy Folders.
 */
class Easy_Folders_Settings
{
    /**
     * Class instance
     * @var $instance
     * @since 1.0.0
    */
    protected static $instance = null;

    /**
     * Folder table
     * @var $folder_table
     * @since 1.0.0
    */
    private static $folder_table = 'efs';

    /**
     * Upload file types
     * @var $upload_file_types
     * @since 1.0.0
    */
    private static $upload_file_types = 'efs_upload_file_types';

    /**
     * Colors
     * @var $efs_colors
     * @since 1.0.0
    */
    private static $efs_colors = 'efs_colors';

    /**
     * Return class instance
     * @since 1.0.0
     * @return object
     */
    public static function getInstance()
    {
        if (null == self::$instance) {
            self::$instance = new self();
        }
        return self::$instance;
    }

    /**
     * Return Easy Folders settings
     * @since 1.0.0
     * @return JSON
     */
    public static function loadSettings()
    {
        global $wpdb;

        $settings = wp_cache_get(EASY_FOLDERS_PREFIX . 'settings', EASY_FOLDERS_CACHE_PREFIX);

        if (!$settings) {

            $options = Easy_Folders_Helper::settingsOptions();
            
            $result = array();

            foreach ($options as $option) {
                if(in_array($option, ['site_status', 'rename_media_type_values', 'rename_media_remove_char', 'media_hover_detail_list'])) {
                    $value = get_option(EASY_FOLDERS_PREFIX. $option);
                    $result[$option] = $value ? $value : [];
                } else {
                    $result[$option] = get_option(EASY_FOLDERS_PREFIX. $option, '');
                }
            }

            //Folders support for custom post types
            $post_types = get_option(EASY_FOLDERS_PREFIX . 'custom-post-types', []);
           
            if($post_types && is_array($post_types) && count($post_types) > 0) {
                foreach ($post_types as $key => $post_type) {
                    $result[$post_type] = get_option(EASY_FOLDERS_PREFIX . $post_type, 0);
                }
            }

            

            //Folder colors
            $efs_colors = $wpdb->prefix . 'efs_colors';

            if ($wpdb->get_var("show tables like '$efs_colors'") == $efs_colors) {

                $folder_colors = $wpdb->get_results($wpdb->prepare('SELECT * FROM %1$s', self::getTable(self::$efs_colors)), ARRAY_A);

                $result['folder_colors'] = (array)$folder_colors;
                
            }

            //Options
            $rating_hide = get_option( EASY_FOLDERS_PREFIX . 'rating-hide' );

            $result['is_rating'] = !$rating_hide ? true : false;

            $wocommerce_sync_hide = get_option( EASY_FOLDERS_PREFIX . 'woocommerce-sync-hide' );

            $result['is_wocommerce_sync'] = $result['woocommerce_support'] && !$wocommerce_sync_hide && class_exists( 'WooCommerce' ) ? true : false;

            $result['installation_steps'] = get_option( EASY_FOLDERS_PREFIX . 'installation-steps', array() );
            
            $result['is_newsletter_widget'] = get_option( EASY_FOLDERS_PREFIX . 'newsletter-hide', false ) || get_option( EASY_FOLDERS_PREFIX . 'newsletter-id');

            $result['is_usage_tracking'] = get_option( EASY_FOLDERS_PREFIX . 'usage-tracking-hide', false );

            $result['is_installed'] = get_option( EASY_FOLDERS_PREFIX . 'install-hide', false );

            $result['is_upgrade_hello_bar'] = !get_option( EASY_FOLDERS_PREFIX . 'upgrade-hellobar-hide' );
            
            $result['is_joy_ride'] = !get_option( EASY_FOLDERS_PREFIX . 'joy-ride-alert-hide' );
            
            //save settings in cache
            wp_cache_add(EASY_FOLDERS_PREFIX . 'settings', $result, EASY_FOLDERS_CACHE_PREFIX);

            return $result;
        } 
        
        return $settings;
    }

    /**
     * Save settings
     * @since 1.0.0
     * @param  mixed $data
     * @return boolean
    */
    public static function saveSettings($data)
    {
        global $wpdb;

        //Clean cache
        wp_cache_delete( EASY_FOLDERS_PREFIX . 'settings', EASY_FOLDERS_CACHE_PREFIX );

        $load_settings = self::loadSettings();

        $options = Easy_Folders_Helper::settingsOptions();
        
        foreach ($options as $key => $option) {
            if(isset($data[$option])) {
                update_option(EASY_FOLDERS_PREFIX . $option, $data[$option]);
            }
        }

        //Folders support for custom post types
        $post_types = get_option(EASY_FOLDERS_PREFIX . 'custom-post-types', []);

        if($post_types && is_array($post_types) && count($post_types) > 0) {
            foreach ($post_types as $key => $post_type) {
                if(isset($data[$post_type])) {
                    Easy_Folders_Helper::addOrUpdateOption(EASY_FOLDERS_PREFIX . $post_type, $data[$post_type]);
                }
            }
        }

        if(isset($load_settings['folder_colors']) && isset($data['folder_colors']) && count((array)$load_settings['folder_colors']) > 0) {
            foreach((array)$load_settings['folder_colors'] as $record) {
                $count = count(array_filter((array)$data['folder_colors'], function($row) use($record) {
                    return isset($row['id']) && $row['id'] == $record['id'];
                }));
                if($count == 0) {
                    $wpdb->delete(self::getTable(self::$efs_colors), array('id' => $record['id']), array('%d'));
                }
            }
        }
        
        if(isset($data['folder_colors']) && count((array)$data['folder_colors']) > 0) {
            foreach($data['folder_colors'] as $input) {
                if(isset($input['id']) && $input['id']) {
                    $wpdb->update(
                        self::getTable(self::$efs_colors),
                        array('color' => $input['color']),
                        array('id' => $input['id']),
                        array('%s'),
                        array('%d')
                    );
                } else {
                    $wpdb->insert(
                        self::getTable(self::$efs_colors),
                        array(
                            'color' => $input['color']
                        )
                    );
                }
            }
        }

        
        
        //Usage tracking
        if($data['usage_tracking'] == 0) {
            Easy_Folders_Helper::addOrUpdateOption(EASY_FOLDERS_PREFIX . 'usage-tracking-disable-timestamp', time());
        }

        //If usage tracking enabled
        if(isset($data['usage_tracking']) && isset($load_settings['usage_tracking']) && $data['usage_tracking'] != $load_settings['usage_tracking'] && $data['usage_tracking'] == 1) {
            //Save usage tracking
            Easy_Folders_Settings::saveUsageTracking(['current_status' => 'Active'], true);

            //Current user who installing plugin
		    Easy_Folders_Helper::addOrUpdateOption(EASY_FOLDERS_PREFIX . 'activating_user', get_current_user_id());
        }

        

        //Clean cache
        wp_cache_delete( EASY_FOLDERS_PREFIX . 'settings', EASY_FOLDERS_CACHE_PREFIX );
        
        return true;
    }

    

    

    /**
     * Load upload file types
     * @since 1.0.0
     * @return object
    */
    public static function loadUploadFileTypes()
    {
        global $wpdb;
        $file_types = wp_cache_get(EASY_FOLDERS_PREFIX . 'upload_file_types', EASY_FOLDERS_CACHE_PREFIX);
        if (!$file_types) {
            $file_types = $wpdb->get_results($wpdb->prepare('SELECT * FROM %1$s', self::getTable(self::$upload_file_types)), ARRAY_A);
            wp_cache_add(EASY_FOLDERS_PREFIX . 'upload_file_types', $file_types, EASY_FOLDERS_CACHE_PREFIX);
            return $file_types;
        }
        return $file_types;
    }

    

    /**
     * Re-create missing database tables
     * @since 1.0.0
     * @return void
    */
    public static function reCreateTable()
    {
        //Create DB Structure
        Easy_Folders_Helper::createDbStructure();
    }

    /**
     * Wipe all data and reset to default factory settings.
     * @since 1.0.0
     * @return boolean
    */
    public static function clearAllData()
    {
        global $wpdb;

        if ($wpdb->get_var($wpdb->prepare('SHOW TABLES LIKE %s', $wpdb->esc_like(self::getTable(self::$folder_table)))) == self::getTable(self::$folder_table)) {

            //Delete folder data
            Easy_Folders_Media::destroyAll();

            $wpdb->query("DELETE FROM " . self::getTable(self::$upload_file_types));

            update_option('easyfolders_updated_from_enhanced', '0');
            update_option('easyfolders_updated_from_wpmlf', '0');
            update_option('easyfolders_updated_from_wpmf', '0');
            update_option('easyfolders_updated_from_realmedia', '0');
            update_option('easyfolders_updated_from_happyfiles', '0');
            update_option('easyfolders_updated_from_filebird', '0');
            update_option('easyfolders_updated_from_wicked_folders', '0');
            update_option('easyfolders_updated_from_premio_folders', '0');

            //Reset default settings
            Easy_Folders_Helper::resetDefaultSettings();

            //Insert default data for folders
            $starred = $wpdb->get_row($wpdb->prepare('SELECT * FROM %1$s WHERE `name` = "Starred"', self::getTable(self::$folder_table)));

            if (is_null($starred)) {
                $wpdb->insert(
                    self::getTable(self::$folder_table),
                    array(
                        'id' => 1,
                        'name' => 'Starred',
                    ),
                    array('%d', '%s')
                );
            }

            $trashed = $wpdb->get_row($wpdb->prepare('SELECT * FROM %1$s WHERE `name` = "Trashed"', self::getTable(self::$folder_table)));

            if (is_null($trashed)) {
                $wpdb->insert(
                    self::getTable(self::$folder_table),
                    array(
                        'id' => 2,
                        'name' => 'Trashed',
                    ),
                    array('%d', '%s')
                );
            }

            //Clean cache
            wp_cache_delete( EASY_FOLDERS_PREFIX . 'settings', EASY_FOLDERS_CACHE_PREFIX );

            return true;
            
        } else {
            return false;
        }
    }

    /**
     * Update specific column for settings
     * @since 1.0.0
     * @param  mixed $column
     * @param  mixed $value
     * @return boolean
    */
    public static function updateSetting($column, $value)
    {
        update_option($column, $value);

        //Clean cache
        wp_cache_delete( EASY_FOLDERS_PREFIX . 'settings', EASY_FOLDERS_CACHE_PREFIX );

        return true;
    }

    /**
     * Return your system's status and see the details of key components.
     * @since 1.0.0
     * @return Array
    */
    public static function getSiteHealth()
    {
        try {
            require_once(ABSPATH . 'wp-admin/includes/update.php');

            require_once(ABSPATH . 'wp-admin/includes/misc.php');

            require_once(ABSPATH . 'wp-admin/includes/screen.php');

            // Load WP_Debug_Data class
            if (!class_exists('WP_Debug_Data')) {
                require_once(ABSPATH . 'wp-admin/includes/class-wp-debug-data.php');
            }

            return WP_Debug_Data::debug_data();
        } catch (Exception $e) {
            return [];
        }
    }

    /**
     * Site health [Copy to clipboard data]
     * @since 1.0.0
     * @param  mixed $response
     * @return string
    */
    public static function getSiteHealthCopyToClipboardData($response)
    {
        $text = "";
        if (count($response) > 0) {
            foreach ($response as $section => $details) {
                $text .= "###" . $section . "####\r\n \r\n \r\n";
                if (count($details['fields']) > 0) {
                    foreach ($details['fields'] as $key => $row) {
                        $text .= $key . ": " . $row['value'] . "\r\n";
                    }
                }
                $text .= "\r\n \r\n";
            }
        }
        return $text;
    }

    /**
     * Submit feedback
     * @since 1.0.0
     * @param  mixed $rating
     * @return boolean
    */
    public static function submitFeedback($rating)
    {
        $response = get_option( EASY_FOLDERS_PREFIX . 'rating' );
        
		if ( $response ) {
			return false;
		}

        update_option( EASY_FOLDERS_PREFIX . 'rating', (int)$rating );

        update_option( EASY_FOLDERS_PREFIX . 'rating-hide', 1 );

        return true;
    }

    /**
     * Delete feedback
     * @since 1.0.0
     * @return void
    */
    public static function deleteFeedback()
    {
        delete_option(EASY_FOLDERS_PREFIX . 'rating');
        delete_option(EASY_FOLDERS_PREFIX . 'rating-hide');
    }

    /**
     * Dismiss notice
     * @since 1.0.0
     * @param  mixed $notice
     * @return void
    */
    public static function dismissNotice($notice, $value = null)
    {
        update_option($notice, true );

        $attempt = get_option($notice.'-count');

        update_option($notice.'-count', $attempt ? ($attempt + 1) : 1 );

        //Tour guide notice
        if($notice == EASY_FOLDERS_PREFIX . 'joy-ride-tour-completed') {
            update_option(EASY_FOLDERS_PREFIX . 'joy-ride-alert-hide', true );
        }
    }

    /**
     * Return table
     * @since 1.0.0
     * @param  mixed $table
     * @return string
    */
    private static function getTable($table)
    {
        global $wpdb;
        return $wpdb->prefix . $table;
    }

    /**
     * Save folder colors
     * @since 1.0.0
     * @param  mixed $color
     * @param  mixed $action
     * @param  mixed $id
     * @return void
    */
    public static function saveColor($color, $action = "add", $id = null)
    {
        global $wpdb;

        if($action == "update") {
            $record =  $wpdb->get_row($wpdb->prepare('SELECT * FROM %1$s WHERE `id` = %2$d', self::getTable(self::$efs_colors), $id));

            if ($record) {
                $wpdb->update(
                    self::getTable(self::$efs_colors),
                    array('color' => $color),
                    array('id' => $id),
                    array('%s'),
                    array('%d')
                );
            }
        } else {
            $wpdb->insert(
                self::getTable(self::$efs_colors),
                array(
                    'color' => $color
                )
            );
        }
        
    }

    /**
     * Delete folder color
     * @since 1.0.0
     * @param  mixed $id
     * @return void
    */
    public static function deleteColor($id)
    {
        global $wpdb;
        $wpdb->delete(self::getTable(self::$efs_colors), array('id' => $id), array('%d'));
    }

    /**
     * Reset default colors
     * @since 1.0.0
     * @return void
    */
    public static function resetDefaultColors()
    {
        global $wpdb;
        
        $colors = array('#e9c445', '#065a93', '#c4694c', '#409692', '#3ac0b1', '#36afd1', '#66b02e', '#74da93', '#7536af', '#563b67', '#8adbbd', '#da1336', '#ff4600', '#000000');

        $folder_colors = (array) $wpdb->get_results($wpdb->prepare('SELECT * FROM %1$s', self::getTable(self::$efs_colors)), ARRAY_A);

        for ($i = (count($folder_colors) + 1); $i <= 14; $i++) { 
            $wpdb->insert(
                self::getTable(self::$efs_colors),
                array(
                    'color' => $colors[$i - 1]
                )
            );
        }
    }

    /**
     * Update installation steps
     * @since 1.0.0
     * @param  mixed $step
     * @return void
    */
    public static function installationStep($step)
    {
        if($step == "skip") {
            $steps = ["initial", "import-data", "connect-account", "subscriptions", "usage_tracking", "ready"];
            update_option('easy-folders-install-hide', true );
        } else {
            $steps = get_option( EASY_FOLDERS_PREFIX . 'installation-steps', array() );
            if(!in_array($step, $steps)) {
                $steps[] = $step;
            }
        }

        update_option(EASY_FOLDERS_PREFIX . 'installation-steps', $steps);

        //Reset connect account notice 
        delete_option(EASY_FOLDERS_PREFIX . 'account-connect-notice');
    }

   /**
     * Return usage tracking data
     * @since 1.0.0
     * @param  mixed $params
     * @return Array
    */
    public static function getUsageTrackingData($params = array())
    {
        $response = Easy_Folders_Settings::getSiteHealth();

        $total = Easy_Folders_Tree::getFolderCount(-1);

        $settings = (array) Easy_Folders_Settings::loadSettings();

        $tree = Easy_Folders_Tree::getAllData('name desc', false);

        $total_folders = Easy_Folders_Tree::getFolderChildrensIds($tree);

        $imported_plugins = array();
        
        if (get_option('easyfolders_updated_from_enhanced', '0') == '1') {
            $imported_plugins[] = 'Enhanced Media Library By wpUXsolutions';
        }
        if (get_option('easyfolders_updated_from_wpmlf', '0') == '1') {
            $imported_plugins[] = 'WordPress Media Library Folders By Max Foundry';
        }
        if (get_option('easyfolders_updated_from_wpmf', '0') == '1') {
            $imported_plugins[] = 'WP Media folder By Joomunited';
        }
        if (get_option('easyfolders_updated_from_realmedia', '0') == '1') {
            $imported_plugins[] = 'WP Real Media Library By devowl.io GmbH';
        }
        if (get_option('easyfolders_updated_from_happyfiles', '0') == '1') {
            $imported_plugins[] = 'HappyFiles by Codeer';
        }
        if (get_option('easyfolders_updated_from_filebird', '0') == '1') {
            $imported_plugins[] = 'FileBird (v4) by NinjaTeam';
        }
        if (get_option('easyfolders_updated_from_wicked_folders', '0') == '1') {
            $imported_plugins[] = 'Wicked Folders by Wicked Plugins';
        }
        if (get_option('easyfolders_updated_from_premio_folders', '0') == '1') {
            $imported_plugins[] = 'Folders by Premio';
        }

        $activating_user_id = get_option(EASY_FOLDERS_PREFIX . 'activating_user');

        $author_obj = get_user_by('id', $activating_user_id);

        $post_data = array(
            'user_profile' => array(
                'first_name' => get_the_author_meta('first_name', $author_obj ? $activating_user_id : 0),
                'country_code' => get_option(EASY_FOLDERS_PREFIX . 'activation-country-code', ''),
                'last_name' => get_the_author_meta('last_name', $author_obj ? $activating_user_id : 0),
                'email' => get_the_author_meta('user_email', $author_obj ? $activating_user_id : 0),
                'username' => get_the_author_meta('user_login', $author_obj ? $activating_user_id : 0),
                'user_language' => isset($response['wp-core']['fields']['user_language']['value']) ? $response['wp-core']['fields']['user_language']['value'] : ''
            ),
            'website_profile' => array(
                'title' => get_bloginfo('name'),
                'admin_email' => get_bloginfo('admin_email'),
                'site_url' => isset($response['wp-core']['fields']['site_url']['value']) ? $response['wp-core']['fields']['site_url']['value'] : '',
                'home_url' => isset($response['wp-core']['fields']['home_url']['value']) ? $response['wp-core']['fields']['home_url']['value'] : '',
                'timezone' => isset($response['wp-core']['fields']['timezone']['value']) ? $response['wp-core']['fields']['timezone']['value'] : '',
                'site_language' => isset($response['wp-core']['fields']['site_language']['value']) ? $response['wp-core']['fields']['site_language']['value'] : '',
                'multisite' => isset($response['wp-core']['fields']['multisite']['value']) ? $response['wp-core']['fields']['multisite']['value'] : '',
                'environment_type' => isset($response['wp-core']['fields']['environment_type']['value']) ? $response['wp-core']['fields']['environment_type']['value'] : '',
                'version' => get_option('easyfolders_version'),
                'plan_id' => isset($settings['site_status']['plan']['name']) ? $settings['site_status']['plan']['id'] : 0,
                'website_id' => isset($settings['site_status']['website']['id']) ? $settings['site_status']['website']['id'] : 0,
                'total_folders_created' => count($total_folders),
                'feedback_id' => get_option(EASY_FOLDERS_PREFIX . 'feedback-id', 0),
                'rating_id' => get_option(EASY_FOLDERS_PREFIX . 'rating-id', 0),
                'newsletter_id' => get_option(EASY_FOLDERS_PREFIX . 'newsletter-id', 0),
                'is_tour_guide' => get_option(EASY_FOLDERS_PREFIX . 'joy-ride-tour-completed', false) ? 1 : 0,
                'current_plugin_status' => isset($params['current_status']) ? $params['current_status'] : '',
                'active_timestamp' => get_option(EASY_FOLDERS_PREFIX . 'account-activation-timestamp', 0),
                'deactive_timestamp' => get_option(EASY_FOLDERS_PREFIX . 'account-deactivation-timestamp', 0),
                'uninstall_timestamp' => get_option(EASY_FOLDERS_PREFIX . 'account-uninstall-timestamp', 0),
                'usage_tracking_disable_timestamp' => get_option(EASY_FOLDERS_PREFIX . 'usage-tracking-disable-timestamp'),
                'subscription_upgrade_timestamp' => get_option(EASY_FOLDERS_PREFIX . 'subscription-upgrade-timestamp'),
                'subscription_downgrade_timestamp' => get_option(EASY_FOLDERS_PREFIX . 'subscription-downgrade-timestamp'),
                'account_connection_timestamp' => get_option(EASY_FOLDERS_PREFIX . 'account-connection-timestamp'),
                'data_imported_plugins' => (array) $imported_plugins,
                'installer_step' => (get_option(EASY_FOLDERS_PREFIX . 'install-hide') ? 'completed' : (get_option(EASY_FOLDERS_PREFIX . 'installation-steps') ? end(get_option(EASY_FOLDERS_PREFIX . 'installation-steps')) : 'uncompleted'))
            ),
            'system_profile' => array(
                'wordpress_version' => isset($response['wp-core']['fields']['version']['value']) ? $response['wp-core']['fields']['version']['value'] : '',
                'webserver' => isset($response['wp-server']['fields']['httpd_software']['value']) ? $response['wp-server']['fields']['httpd_software']['value'] : '',
                'php_version' => isset($response['wp-server']['fields']['php_version']['value']) ? $response['wp-server']['fields']['php_version']['value'] : '',
                'database_extension' => isset($response['wp-database']['fields']['extension']['value']) ? $response['wp-database']['fields']['extension']['value'] : '',
                'database_server_version' => isset($response['wp-database']['fields']['server_version']['value']) ? $response['wp-database']['fields']['server_version']['value'] : '',
            ),
            'theme_profile' => array(
                'current_theme' => isset($response['wp-active-theme']['fields']['name']['value']) ? $response['wp-active-theme']['fields']['name']['value'] : '',
                'version' => isset($response['wp-active-theme']['fields']['version']['value']) ? $response['wp-active-theme']['fields']['version']['value'] : '',
                'auto_update' => isset($response['wp-active-theme']['fields']['auto_update']['value']) ? $response['wp-active-theme']['fields']['auto_update']['value'] : '',
                'themes' => isset($response['wp-themes-inactive']['fields']) ? $response['wp-themes-inactive']['fields'] : [],
            ),
            'plugins_profile' => array(
                'active' => isset($response['wp-plugins-active']['fields']) ? $response['wp-plugins-active']['fields'] : [],
                'inactive' => isset($response['wp-plugins-inactive']['fields']) ? $response['wp-plugins-inactive']['fields'] : [],
            ),
            'media_profile' => array(
                'active_media_editor' => isset($response['wp-media']['fields']['image_editor']['value']) ? $response['wp-media']['fields']['image_editor']['value'] : '',
                'media_count' => $total,
            ),
        );

        return $post_data;
    }
    
    /**
     * Save usage tracking
     * @since 1.0.0
     * @return void
    */
    public static function saveUsageTracking($default_data = array(), $call = false)
    {
        $settings = (array) Easy_Folders_Settings::loadSettings();

        if($settings['usage_tracking'] == 1 || $call) {

            $data = Easy_Folders_Settings::getUsageTrackingData();

            $data = !empty($default_data) ? array_merge($default_data, $data) : $data;

            $api = new Easy_Folders_Api(Easy_Folders_Helper::getAccountApikey());
    
            $api->saveUsageTracking($data, get_site_url(), true);
        }
    }
        
    /**
     * Save site health/api status
     * @since 1.0.0
     * @param  mixed $settings
     * @param  mixed $status
     * @return void
    */
    public static function saveSiteStatus($settings, $status)
    {
        
    }

    /**
     * Remove credentials
     * @since 1.0.0
     * @return void
    */
    public static function removeCredentials()
    {
        update_option(EASY_FOLDERS_PREFIX .'site_status', '');

        update_option(EASY_FOLDERS_PREFIX .'license_key', '');

        //Clean cache
        wp_cache_delete( EASY_FOLDERS_PREFIX . 'settings', EASY_FOLDERS_CACHE_PREFIX );
    }
        
    /**
     * Return localize script json for JS
     * @since 1.0.0
     * @param  mixed $screen
     * @param  mixed $settings
     * @return Array
    */
    public static function getLocalizeScript($screen, $settings) {

        $post_type = Easy_Folders_Helper::get_custom_post_type();

        $media_mode = $post_type != 'attachment' ? 'list' : (in_array($screen, ['media-new.php']) ? 'grid' : get_user_option('media_library_mode', get_current_user_id()));

        $localize_array = array(
            'admin_url' => admin_url('admin.php'),
            'post_url' => admin_url('post.php'),
            'apiUrl'    => apply_filters('easyfolders_json_url', rtrim(rest_url(EASY_FOLDERS_REST_API_PREFIX), "/")),
            'easyfolders_go_url'    => EASY_FOLDERS_GO_URL,
            'nonce'   => wp_create_nonce('wp_rest'),
            'rest_nonce' => wp_create_nonce('wp_rest'),
            'nonce_error' => __('Your request can\'t be processed.', 'easy-folders'),
            'i18n' => Easy_Folders_i18n::getTranslation(),
            'media_mode' => $media_mode,
            'media_url' => admin_url('upload.php'),
            'ajaxUrl'             => admin_url('admin-ajax.php'),
            'async_upload_url' => admin_url('async-upload.php'),
            'media_nonce'      => wp_create_nonce('media-form'),
            'screen' => $screen,
            'pluginUrl'               => EASY_FOLDERS_PLUGIN_URL,
            'siteUrl'               => get_site_url(),
            'settings' => $settings,
            'plan' => Easy_Folders_Subscription::is_pro($settings) ? 'pro' : 'free',
            'plan_and_plugin' => Easy_Folders_Subscription::is_pro($settings) && is_plugin_active( 'easy-folders-pro/easy-folders.php' ) && EASY_FOLDERS_BASENAME == 'easy-folders-pro/easy-folders.php' ? 'pro' : 'free',
            'is_download_pro' => (is_plugin_active( 'easy-folders/easy-folders.php' ) && Easy_Folders_Subscription::is_pro()) && !is_plugin_active( 'easy-folders-pro/easy-folders.php' ) ? 1 : 0,
            'is_pro' => is_plugin_active( 'easy-folders-pro/easy-folders.php' ) && EASY_FOLDERS_BASENAME == 'easy-folders-pro/easy-folders.php' ? 1 : 0,
            'current_user_id' => get_current_user_id(),
            'restUrl' => get_rest_url(),
            'is_administrator' => is_admin() && is_super_admin(),
            'is_rtl' => is_rtl(),
            'post_type' => $post_type,
            'css' => array(
                'left' => is_rtl() ? 'right' : 'left',
                'right' => is_rtl() ? 'left' : 'right'
            ),
            'current_user' => array(
                'name' => Easy_Folders_Helper::getUserName()
            ), 
            'website_profile' => array(
                'first_name' => ucfirst(get_the_author_meta('first_name', get_current_user_id())),
                'last_name' => ucfirst(get_the_author_meta('last_name', get_current_user_id())),
                'admin_email' => get_the_author_meta('user_email', get_current_user_id()),
                'admin_username' => ucfirst(get_the_author_meta('user_login', get_current_user_id())),
            )
        );

        $upgrade_url = add_query_arg(
            array(
                'utm_source'   => 'easy-folders',
                'utm_medium'   => 'plugin',
                'utm_campaign' => 'easyfolders_bulkeasyfolders_issues_filesizelimit_notice',
            ),
            EASY_FOLDERS_GO_URL.'/pricing'
        );

        $final_data = array_merge($localize_array, array(
            'folders' => Easy_Folders_Media::getAllFolders('id as term_id, name as term_name', array('term_id', 'term_name')),
            'sort_folder' => get_option('easyfolders_sort_folder', 'reset'),
            'attachmentsBrowser'  => '',
            'filterId'  => 'custom-search-filter',
            'filterBy'             => 'efs',
            'terms'               => Easy_Folders_Tree::getAllFoldersId("name asc"),
            'data'               => Easy_Folders_Media_Controller::getAllData($post_type),
            'trash'               => MEDIA_TRASH,
            'query' => get_option('efs-query-params-'.$post_type) && ($media_mode === "grid" || isset($_GET['efs'])) ? get_option('efs-query-params-'.$post_type) : []
        ));

        return $final_data;
    }
    
    /**
     * This is to detect wordpress default/active translation language
     * @since 1.0.0
     * @return string
     */
    public static function getActiveTranslation() {
        $active_locale = get_locale();
		if ( 'en' === $active_locale || 'en_US' === $active_locale ) {
			$language = 'English';
		} else {
			require_once ABSPATH . 'wp-admin/includes/translation-install.php';
			$available_translations  = wp_get_available_translations();
			$language = isset( $available_translations[ $active_locale ] ) ? $available_translations[ $active_locale ]['native_name'] : __( 'No language detected', 'easy-folders' );
		}
        return $language;
    }
}
