<?php
trait Easy_Folders_Notice_Hooks {

	/**
	 * List of easy-folders settings pages.
	 * @since 1.0.0
	 * @var array $plugin_pages
	 */
	public static $plugin_pages = array(
		'toplevel_page_easyfolders',
		'easyfolders_page_easyfolders-pro',
		'upload'
	);
	
    /**
	 * Shows Notice for free users, displays a discount coupon
	 * Initial trial upgrade notice 
	 * (Easy Folders pages)
	 * @since 1.0.0
	 * @return void
	 */
	public function initial_trial_upgrade_notice() {

		// Return, If a pro user, or not super admin, or don't have the administrator.
		if (  Easy_Folders_Subscription::is_pro() || ! is_admin() || ! is_super_admin()) {
			return;
		}

		//Don't display notice on installer page
		if(!get_option(EASY_FOLDERS_PREFIX . 'install-hide')) {
			return;
		}

		// Return if notice is already dismissed.
		if ( get_site_option( EASY_FOLDERS_PREFIX . 'initial-trial-upgrade-notice' ) ) {
			return;
		}

		// Show it on Easy Folders pages only.
		$screen = function_exists('get_current_screen') ? get_current_screen() : false;

		if(is_plugin_active( 'easy-folders/easy-folders.php' ) && EASY_FOLDERS_BASENAME == 'easy-folders/easy-folders.php' && in_array( $screen->id, self::$plugin_pages, true )) {

			$message = __( 'Thanks for installing Easy Folders! %1$sGet a free trial of Easy Folders Pro now%2$s and unlock true power of Easy Folders. Grab it while it lasts.', 'easy-folders' );

			$upgrade_url = add_query_arg(
				array(
					'utm_source'   => 'easy-folders',
					'utm_medium'   => 'plugin',
					'utm_campaign' => 'easyfolders_initial_trial_upgrade_notice',
				),
				EASY_FOLDERS_GO_URL.'/pricing'
			);
	
			$this->loadTemplate('notices/initial-trial-upgrade-notice', [
				'upgrade_url' => $upgrade_url,
				'message' => $message,
			]);
			
		}
		
	}

    /**
	 * Display a admin notice about free to pro plugin conversion. 
	 * (Easy Folders pages & plugins)
	 * @since 1.0.0
	 * @return void
	 */
	public function free_to_pro_plugin_conversion_notice() {
		
		//If free plugin is active enable notice
		if ( is_plugin_active( 'easy-folders/easy-folders.php' ) && EASY_FOLDERS_BASENAME == 'easy-folders/easy-folders.php' ) {
			delete_option(  EASY_FOLDERS_PREFIX . 'free-to-pro-plugin-conversion-notice' );
		}

		
	}

	

    /**
	 * Display plugin incompatibility notice.
	 * (Easy Folders pages & plugins)
	 * @since 1.0.0
	 * @return void
	 */
	public function show_plugin_conflict_notice() {

		// Display only in backend for administrators.
		if ( ! is_admin() || ! is_super_admin() ) {
			return;
		}

		//Don't display notice on installer page
		if(!get_option(EASY_FOLDERS_PREFIX . 'install-hide')) {
			return;
		}

		$dismissed = get_option( EASY_FOLDERS_PREFIX . 'conflict-notice' );
        
		if ( $dismissed ) {
			return;
		}

		$plugins = get_option( EASY_FOLDERS_PREFIX . 'conflict_check' );

		// Have never checked before.
		if (false === $plugins || empty($plugins) || $plugins == null) {
			return;
		}

		// No conflicting plugins detected.
		if ( isset( $plugins ) && is_array( $plugins ) && empty( $plugins ) ) {
			return;
		}

		array_walk(
			$plugins,
			function( &$item ) {
				$item = '<strong>' . $item . '</strong>';
			}
		);
		
		// Show it on Easy Folders pages only & plugin page.
		$screen = function_exists('get_current_screen') ? get_current_screen() : false;

		if(in_array( $screen->id, self::$plugin_pages, true ) || $screen->id == 'plugins') {
			$this->loadTemplate('notices/show-plugin-conflict-notice', [
				'plugins' => $plugins
			]);
		}
		
	}
	
	/**
	 * Upgrade to pro
	 * (Easy Folders pages & plugins)
	 * @since 1.0.0
	 * @return void
	 */
	public function upgrade_to_pro_notice() {

		// Display only in backend for administrators.
		if (Easy_Folders_Subscription::is_pro() || ! is_admin() || ! is_super_admin() ) {
			return;
		}
		
		//Don't display notice on installer page
		if(!get_option(EASY_FOLDERS_PREFIX . 'install-hide')) {
			return;
		}

		$dismissed = get_option( EASY_FOLDERS_PREFIX . 'upgrade-to-pro-notice' );
        
		if ( $dismissed ) {
			return;
		}

		$message = __('Start a free trial to unlock all the features of the Media library.', 'easy-folders');
		
		$button_text = __('Start free trial', 'easy-folders');

		// Show it on Easy Folders pages only & plugin page.
		$screen = function_exists('get_current_screen') ? get_current_screen() : false;

		if(in_array( $screen->id, self::$plugin_pages, true ) || $screen->id == 'plugins') {
			$this->loadTemplate('notices/upgrade-to-pro-notice', [
				'message' => $message,
				'button_text' => $button_text
			]);
		}
	}

	/**
	 * Display plugin usage tracking notice.
	 * (Easy Folders pages)
	 * @since 1.0.0
	 * @return void
	 */
	public function show_plugin_usage_tracking_notice() {

		// Display only in backend for administrators.
		if ( ! is_admin() || ! is_super_admin() ) {
			return;
		}

		//Don't display notice on installer page
		if(!get_option(EASY_FOLDERS_PREFIX . 'install-hide')) {
			return;
		}
		
		$dismissed = get_option( EASY_FOLDERS_PREFIX . 'usage-tracking-notice-hide' );
        
		if ( $dismissed || $this->settings['usage_tracking'] == 1) {
			return;
		}

		// Show it on Easy Folders pages only.
		$screen = function_exists('get_current_screen') ? get_current_screen() : false;

		if(in_array( $screen->id, self::$plugin_pages, true )) {
			$this->loadTemplate('notices/plugin-usage-tracking-notice', [
				'username' => Easy_Folders_Helper::getUserName()
			]);
		}
	}

	/**
	 * Display Easy Folders rate notice.
	 * (Easy Folders pages)
	 * @since 1.0.0
	 * @return void
	 */
	public function easyfolders_rate_notice() {

		// Display only in backend for administrators.
		if ( ! is_admin() || ! is_super_admin() ) {
			return;
		}

		//Don't display notice on installer page
		if(!get_option(EASY_FOLDERS_PREFIX . 'install-hide')) {
			return;
		}
		
		$dismissed = get_option( EASY_FOLDERS_PREFIX . 'rate-notice-hide' );
        
		if ( $dismissed) {
			return;
		}

		//If already done
		$already_done = get_option( EASY_FOLDERS_PREFIX . 'rate-notice-already-done' );

		if ( $already_done) {
			return;
		}

		// Show it on Easy Folders pages only.
		$screen = function_exists('get_current_screen') ? get_current_screen() : false;

		if(in_array( $screen->id, self::$plugin_pages, true )) {
			$this->loadTemplate('notices/easy-folders-rate-notice');
		}
		
	}
	
	/**
	 * Dismiss notices according to types.
	 * @since 1.0.0
	 * @return void
	 */
	public function dismiss_notice() {
		
		if(isset($_POST['delete']) && $_POST['delete']) {
			delete_option( trim(sanitize_text_field($_POST['notice'])));
		} else {
			update_option( trim(sanitize_text_field($_POST['notice'])), true );
		}
		
		wp_send_json_success();
	}

	/**
     * Check for plugin conflicts cron.
     * @since 1.0.0
     * @param string $deactivated  Holds the slug of activated/deactivated plugin.
	 * @return void
     */
	public function check_for_conflicts_cron( $deactivated = '' ) {

		$conflicting_plugins = array(
			'folders/folders.php',
			'filebird-pro/filebird.php',
			'filebird/filebird.php',
			'real-media-library/index.php',
			'real-media-library-lite/index.php',
			'enable-media-replace/enable-media-replace.php',
			'happyfiles/happyfiles.php',
			'wp-media-folder/wp-media-folder.php',
			'media-library-plus/media-library-plus.php',
			'wp-media-manager-lite/wp-media-manager-lite.php',
			'organize-media-folder/organizemediafolder.php',
			'wp-media-folders/wp-media-folders.php',
			'catfolders/catfolders.php',
			'media-folders-lite/media-folders.php',
			'media-folders/media-folders.php',
			'media-library-organizer/media-library-organizer.php',
			'enhanced-media-library/enhanced-media-library.php',
			'wp-media-category-management/wp-media-category-management.php',
			'wicked-folders/wicked-folders.php'
		);

		$plugins = get_plugins();

		$active_plugins = array();

		foreach ( $conflicting_plugins as $plugin ) {
            
			if ( ! array_key_exists( $plugin, $plugins ) ) {
				continue;
			}

			if ( ! is_plugin_active( $plugin ) ) {
				continue;
			}

			// Deactivation of the plugin in process.
			if ( doing_action( 'deactivated_plugin' ) && $deactivated === $plugin ) {
				continue;
			}

			$active_plugins[] = $plugins[ $plugin ]['Name'];
			
		}

		//Update conflict plugins
		update_option( EASY_FOLDERS_PREFIX . 'conflict_check', $active_plugins );
		
		//Delete dismiss notice for plugin conflicts
		delete_option( EASY_FOLDERS_PREFIX . 'conflict-notice' );
		
	}
}
?>
