<?php

/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 * @link       https://ripebits.com
 * @since      1.0.0
 * @package    EasyFolders
 * @subpackage EasyFolders/includes
 */

/**
 * The core plugin class.
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 * @since      1.0.0
 * @package    EasyFolders
 * @subpackage EasyFolders/includes
 */
class Easy_Folders
{

	/**
	 * The loader that's responsible for maintaining and registering all hooks that power
	 * the plugin.
	 * @since    1.0.0
	 * @access   protected
	 * @var      Easy_Folders_Loader    $loader    Maintains and registers all hooks for the plugin.
	 */
	protected $loader;

	/**
	 * The unique identifier of this plugin.
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $plugin_name    The string used to uniquely identify this plugin.
	 */
	protected $plugin_name;

	/**
	 * The current version of the plugin.
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $version    The current version of the plugin.
	 */
	protected $version;

	/**
	 * Define the core functionality of the plugin.
	 * Set the plugin name and the plugin version that can be used throughout the plugin.
	 * Load the dependencies, define the locale, and set the hooks for the admin area and
	 * the public-facing side of the site.
	 * @since    1.0.0
	 * @return void
	 */
	public function __construct()
	{
		if (defined('EASY_FOLDERS_VERSION')) {
			$this->version = EASY_FOLDERS_VERSION;
		} else {
			$this->version = '1.0.0';
		}

		$this->plugin_name = EASY_FOLDERS_PLUGIN_NAME;

		$this->load_dependencies();

		$this->set_locale();

		$this->define_admin_hooks();

		$this->define_public_hooks();
	}

	/**
	 * Load the required dependencies for this plugin.
	 * Include the following files that make up the plugin:
	 * - Easy_Folders_Loader. Orchestrates the hooks of the plugin.
	 * - Easy_Folders_i18n. Defines internationalization functionality.
	 * - Easy_Folders_Admin. Defines all hooks for the admin area.
	 * - Easy_Folders_Public. Defines all hooks for the public side of the site.
	 * Create an instance of the loader which will be used to register the hooks
	 * with WordPress.
	 * @since    1.0.0
	 * @access   private
	 * @return void
	 */
	private function load_dependencies()
	{
		/**
		 * The class responsible for orchestrating the actions and filters of the
		 * core plugin.
		 */
		require_once plugin_dir_path(dirname(__FILE__)) . 'includes/class-easy-folders-loader.php';

		/**
		 * The class responsible for defining internationalization functionality
		 * of the plugin.
		 */
		require_once plugin_dir_path(dirname(__FILE__)) . 'includes/class-easy-folders-i18n.php';

		/**
		 * The class/traits responsible for defining all actions that occur in the admin area.
		 */
		require_once plugin_dir_path(dirname(__FILE__)) . 'admin/traits/class-easy-folders-notice-hooks.php';
		require_once plugin_dir_path(dirname(__FILE__)) . 'admin/traits/class-easy-folders-general-hooks.php';
		require_once plugin_dir_path(dirname(__FILE__)) . 'admin/traits/crons/class-easy-folders-crons-hooks.php';

		/**
		 * The class responsible for defining all actions that occur in the public-facing
		 * side of the site.
		 */
		require_once plugin_dir_path(dirname(__FILE__)) . 'public/class-easy-folders-public.php';

		//Helpers
		require_once plugin_dir_path(dirname(__FILE__)) . 'admin/helpers/class-easy-folders-helper.php';
		require_once plugin_dir_path(dirname(__FILE__)) . 'admin/helpers/class-easy-folders-iterator.php';

		//Media
		require_once plugin_dir_path(dirname(__FILE__)) . 'admin/repositories/class-easy-folders-tree.php';
		require_once plugin_dir_path(dirname(__FILE__)) . 'admin/repositories/media/class-easy-folders-media.php';
		require_once plugin_dir_path(dirname(__FILE__)) . 'admin/controllers/class-easy-folders-media-controller.php';
		require_once plugin_dir_path(dirname(__FILE__)) . 'admin/repositories/page-builder/class-easy-folders-page-builder.php';
		require_once plugin_dir_path(dirname(__FILE__)) . 'admin/hooks/class-easy-folders-media-hook.php';

		//Settings
		require_once plugin_dir_path(dirname(__FILE__)) . 'admin/repositories/class-easy-folders-menus.php';
		require_once plugin_dir_path(dirname(__FILE__)) . 'admin/repositories/import/class-easy-folders-import.php';
		require_once plugin_dir_path(dirname(__FILE__)) . 'admin/controllers/class-easy-folders-import-controller.php';
		require_once plugin_dir_path(dirname(__FILE__)) . 'admin/controllers/class-easy-folders-settings-controller.php';
		require_once plugin_dir_path(dirname(__FILE__)) . 'admin/repositories/class-easy-folders-settings.php';

		//Api handling
		require_once plugin_dir_path(dirname(__FILE__)) . 'admin/repositories/api/class-easy-folders-request.php';
		require_once plugin_dir_path(dirname(__FILE__)) . 'admin/repositories/api/class-easy-folders-api.php';

		//Common
		require_once plugin_dir_path(dirname(__FILE__)) . 'admin/repositories/class-easy-folders-subscription.php';

		//Integrations
		require_once plugin_dir_path(dirname(__FILE__)) . 'admin/repositories/gutenberg/class-easy-folders-admin-gutenberg.php';

		//Svg support
		require_once plugin_dir_path(dirname(__FILE__)) . 'admin/repositories/svg/class-easy-folders-svg-tags.php';
		require_once plugin_dir_path(dirname(__FILE__)) . 'admin/repositories/svg/class-easy-folders-svg-attributes.php';
		require_once plugin_dir_path(dirname(__FILE__)) . 'admin/repositories/svg/class-easy-folders-svg-support.php';

		if ( ! function_exists( 'wp_handle_upload' ) ) {
			require_once(ABSPATH . 'wp-admin/includes/file.php');
		}
		
		if (!function_exists('wp_generate_attachment_metadata')) {
			require_once(ABSPATH . 'wp-admin/includes/image.php');
		}

		//Plugin updates
		require_once plugin_dir_path(dirname(__FILE__)) . 'admin/repositories/class-easy-folders-plugin-update.php';

		require_once plugin_dir_path(dirname(__FILE__)) . 'admin/class-easy-folders-admin.php';

		$this->loader = new Easy_Folders_Loader();
	}

	/**
	 * Define the locale for this plugin for internationalization.
	 * Uses the Easy_Folders_i18n class in order to set the domain and to register the hook
	 * with WordPress.
	 * @since    1.0.0
	 * @access   private
	 * @return void
	 */
	private function set_locale()
	{
		$plugin_i18n = new Easy_Folders_i18n();

		$this->loader->add_action('plugins_loaded', $plugin_i18n, 'load_plugin_textdomain');
	}

	/**
	 * Register all of the hooks related to the admin area functionality
	 * of the plugin.
	 * @since    1.0.0
	 * @access   private
	 * @return void
	*/
	private function define_admin_hooks()
	{
		$plugin_admin = new Easy_Folders_Admin($this->get_plugin_name(), $this->get_version());

		//Action hook for loading script
		$this->loader->add_action('admin_enqueue_scripts', $plugin_admin, 'enqueue_styles');
		$this->loader->add_action('admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts');

		//Media hooks
		$this->media_hooks($plugin_admin);

		//Settings hooks
		$this->settings_hooks($plugin_admin);

		//Fires as an admin screen or script is being initialized.
		$this->loader->add_action('admin_init', $plugin_admin, 'admin_init');

		//Fires after WordPress has finished loading but before any headers are sent.
		$this->loader->add_action('init', $plugin_admin, 'init');

		//Cron schedules
		$this->loader->add_action('cron_schedules', $plugin_admin, 'cron_schedules');
		
		//Crons jobs
		
		
		$this->loader->add_action(EASY_FOLDERS_PREFIX . 'delete_zip_files', $plugin_admin, 'deleteZipFiles');
		$this->loader->add_action(EASY_FOLDERS_PREFIX . 'feedback_notice_cron', $plugin_admin, 'feedback_notice_cron');
		$this->loader->add_action(EASY_FOLDERS_PREFIX . 'upgrade_hello_bar_cron', $plugin_admin, 'upgrade_hello_bar_cron');
		$this->loader->add_action(EASY_FOLDERS_PREFIX . 'usage_tracking_modal_cron', $plugin_admin, 'usage_tracking_modal_cron');
		$this->loader->add_action(EASY_FOLDERS_PREFIX . 'post_usage_tracking_cron', $plugin_admin, 'post_usage_tracking_cron');
		$this->loader->add_action(EASY_FOLDERS_PREFIX . 'upgrade_to_pro_cron', $plugin_admin, 'upgrade_to_pro_cron');
		$this->loader->add_action(EASY_FOLDERS_PREFIX . 'daily_cron', $plugin_admin, 'dailyCron');
		$this->loader->add_action(EASY_FOLDERS_PREFIX . 'clean_folders_cron', $plugin_admin, 'clean_folders_cron');

	}

	/**
	 * Register all of the hooks related to the public-facing functionality
	 * of the plugin.
	 * @since    1.0.0
	 * @access   private
	 * @return void
	 */
	private function define_public_hooks()
	{
		$plugin_public = new Easy_Folders_Public($this->get_plugin_name(), $this->get_version());

		$this->loader->add_action('wp_enqueue_scripts', $plugin_public, 'enqueue_styles');

		$this->loader->add_action('wp_enqueue_scripts', $plugin_public, 'enqueue_scripts');
	}

	/**
	 * Run the loader to execute all of the hooks with WordPress.
	 * @since    1.0.0
	 * @return void
	 */
	public function run()
	{
		$this->loader->run();
	}

	/**
	 * The name of the plugin used to uniquely identify it within the context of
	 * WordPress and to define internationalization functionality.
	 * @since     1.0.0
	 * @return    string    The name of the plugin.
	 */
	public function get_plugin_name()
	{
		return $this->plugin_name;
	}

	/**
	 * The reference to the class that orchestrates the hooks with the plugin.
	 * @since     1.0.0
	 * @return    Easy_Folders_Loader    Orchestrates the hooks of the plugin.
	 */
	public function get_loader()
	{
		return $this->loader;
	}

	/**
	 * Retrieve the version number of the plugin.
	 * @since     1.0.0
	 * @return    string    The version number of the plugin.
	 */
	public function get_version()
	{
		return $this->version;
	}

	/**
	 * Media screen hooks
	 * @since     1.0.0
	 * @return    void
	*/
	public function media_hooks($plugin_admin)
	{
		$this->loader->add_action('rest_api_init', $plugin_admin, 'media_routes');

		$this->loader->add_action('add_attachment', $plugin_admin, 'addAttachment');

		$this->loader->add_filter('wp_handle_upload_prefilter', $plugin_admin, 'uploadPreFilter');

		$this->loader->add_action('deleteAttachment', $plugin_admin, 'deleteAttachment');

		$this->loader->add_action('ajax_query_attachments_args', $plugin_admin, 'setQueryAttachmentsArgs', 20);

		$this->loader->add_action('mla_media_modal_query_final_terms', $plugin_admin, 'setQueryAttachmentsArgs', 20);

		$this->loader->add_action('posts_clauses', $plugin_admin, 'customizedPostsClauses', 10, 2);

		$this->loader->add_action('easyfolders_extend_fetch_count_where_query', $plugin_admin, 'extendFetchCountWhereQuery', 10, 1);

		$this->loader->add_filter( 'pre-upload-ui', $plugin_admin, 'upload_ui_media_new' );
	}

	/**
	 * Settings screens hooks
	 * @since     1.0.0
	 * @return    void
	*/
	public function settings_hooks($plugin_admin)
	{
		//Admin menus
		$this->loader->add_action('admin_menu', $plugin_admin, 'settingMenus', 11);

		//Fires in head section for all admin pages.
		$this->loader->add_action('admin_head', $plugin_admin, 'adminHead', 11);

		//routes
		$this->loader->add_action('rest_api_init', $plugin_admin, 'settings_routes');

		//Export easy folders content
		$this->loader->add_action('admin_init', $plugin_admin, 'exportContent');

		//Plugins list screen
		$this->loader->add_action('plugin_row_meta', $plugin_admin, 'plugin_row_meta', 10, 2);
		$this->loader->add_action('plugin_action_links_' . EASY_FOLDERS_BASENAME, $plugin_admin, 'plugin_action_links', 10);

		/**********Notices***********/
		$this->loader->add_action( 'admin_notices', $plugin_admin, 'initial_trial_upgrade_notice');
		$this->loader->add_action( 'network_admin_notices', $plugin_admin, 'initial_trial_upgrade_notice' );

		$this->loader->add_action( 'admin_notices', $plugin_admin, 'free_to_pro_plugin_conversion_notice');
		$this->loader->add_action( 'network_admin_notices', $plugin_admin, 'free_to_pro_plugin_conversion_notice');
		
		
		
		// Plugin conflict notice.
		$this->loader->add_action( 'admin_notices', $plugin_admin, 'show_plugin_conflict_notice' );
		$this->loader->add_action( 'activated_plugin', $plugin_admin, 'check_for_conflicts_cron' );
		$this->loader->add_action( 'deactivated_plugin', $plugin_admin, 'check_for_conflicts_cron' );

		//Account related notice
		$this->loader->add_action( 'admin_notices', $plugin_admin, 'upgrade_to_pro_notice' );

		/**********General hooks***********/
		$this->loader->add_action( 'wp_ajax_dismiss_notice', $plugin_admin, 'dismiss_notice' );

		// Usage tracking notice/settings.
		$this->loader->add_action( 'admin_notices', $plugin_admin, 'show_plugin_usage_tracking_notice' );
		$this->loader->add_action( 'wp_ajax_save_usage_tracking', $plugin_admin, 'save_usage_tracking' );

		//Easy Folders rate notice
		$this->loader->add_action( 'admin_notices', $plugin_admin, 'easyfolders_rate_notice' );
		
		//Plugin activation
		$this->loader->add_action( 'admin_init', $plugin_admin, 'redirect_after_installation' );

		//Fires once activated plugins have loaded.
		$this->loader->add_action('plugins_loaded', $plugin_admin, 'plugins_loaded');
	}
}
